/**
 * 报表块
 * @extends FR.OB
 * 
 * @cfg {JSON} options 配置属性
 * @cfg {Number} options.sheet 报表块所在sheet
 * @cfg {Number} options.sheetIndex 报表块所在sheet的index
 * @cfg {String} options.type 报表块类型（'center','corner','west','north'）
 * @cfg {Number} options.contentWidth 内容宽度
 * @cfg {Number} options.contentHeight 内容宽度
 * @cfg {Boolean} options.isShowWidgets 直接显示控件
 */
WT.ReportBlock = FR.extend(FR.OB, {

    _const: {
        scaleMultiple: 3
    },

	_defaultConfig: function () {
        return $.extend(WT.ReportBlock.superclass._defaultConfig.apply(), {
            sheet: null,
        	sheetIndex: 0,
			type: 'center',
            contentWidth: 0,
            contentHeight: 0,
            isShowWidgets: false,
            sheetContext: null
		});
    },

	_init: function () {
		WT.ReportBlock.superclass._init.apply(this, arguments);
		this.loadStarted = false;  //是否已经开始加载单元格数据了
		this.cellLoaded = false;  //是否已经有单元格已经加载完成并显示过了
		this.cellMap = {};
		this.synchronizeCellDataDelayed = WT.debounce(this._synchronizeCellData, 300);
		this.needToSyncAfterDragStop = false;
		this.scrollLeft = 0;
		this.scrollTop = 0;
        this.actualScrollLeft = 0;
        this.actualScrollTop = 0;
        this.isScrollingOutOfRect = false;
        this.fetchedCells = null;
		this._initView();
	},

	/**
	 * 初始化dom以及子控件dom
	 */
	_initView: function () {
	    var self = this;
        this.element = $('<div class="wt-absolute-layout wt-report-block"></div>');
        if (this.options.type === 'center') {
            this.element.addClass('center');
        }
        else if (this.options.type === 'north') {
            this.element.addClass('north');
        }
        else if (this.options.type === 'west') {
            this.element.addClass('west');
        }
        else {
            this.element.addClass('corner');
        }

        this.placeholder = $('<div class="placeholder"></div>');
        this.placeholder.on('click', function () {
            self.fireEvent(WT.ReportBlock.EVENT_CELL_UNSELECT);
        });
        this.element.append(this.placeholder);
	},

    _handleRectChange: function (type) {
	    if (type === 'wheel') {
            this.synchronizeCellDataDelayed();
        } else if (type === 'drag') {
            this.needToSyncAfterDragStop = true;
        } else {
	        this._synchronizeCellData();
        }
    },

    /**
     * 加载/同步单元格数据
     * @param force 是否强制加载
     * @private
     */
    _synchronizeCellData: function (force) {
        var o = this.options, self = this;
        var lastRect = this.cellRect;
        if (force !== true
            && lastRect
            && lastRect.width === this.size.width
            && lastRect.height === this.size.height) {
            var hThreshold = Math.floor(lastRect.width * this._const.scaleMultiple / 8);
            var vThreshold = Math.floor(lastRect.height * this._const.scaleMultiple / 8);
            if (Math.abs(this.actualScrollLeft - lastRect.left) < hThreshold
                && Math.abs(this.actualScrollTop - lastRect.top) < vThreshold) {
                return;
            }
        }
        if (force !== true && this.showCover) {
            return;
        }
        this.cellRect = {
            left: this.actualScrollLeft,
            top: this.actualScrollTop,
            width: this.size.width,
            height: this.size.height
        };
        if (force === true) {
            this.showCover = true;
            this.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, true);
            /*
            //首次加载时，先只加载窗口大小范围的单元格，然后延迟加载周围更大范围的格子
            WT.net.getCellsByRange(o.sheetIndex, o.type, this.cellRect, o.sheet.element).then(function (data) {
                self._applyCells(data.cells);
                self._scrollPageTo(self.actualScrollLeft, self.actualScrollTop);
            })['catch'](WT.emptyFn).then(function () {
                if (self.showCover) {
                    self.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, false);
                    self.showCover = false;
                }
            }).then(function () {
                if (self.cellRect.width < o.contentWidth || self.cellRect.height < o.contentHeight) {
                    //内容大小比窗口大小更大时，加载周围更大范围的格子
                    return WT.setTimeout(10).then(function () {
                        var resultRect = self._scaleRect(self.cellRect);
                        return WT.net.getCellsByRange(o.sheetIndex, o.type, resultRect, o.sheet.element).then(function (data) {
                            self._applyCells(data.cells);
                        });
                    });
                }
            })['catch'](WT.emptyFn);
            */
            this.fireEvent(WT.ReportBlock.EVENT_LOADING_STATUS, true);
            WT.net.getCellsByRange(o.sheetIndex, o.type, this._scaleRect(this.cellRect), o.sheet.element).then(function (data) {
                self.fetchedCells = data.cells;
            })['catch'](WT.emptyFn).then(function () {
                self.fireEvent(WT.ReportBlock.EVENT_LOADING_STATUS, false);
                if (self.showCover) {
                    self.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, false);
                    self.showCover = false;
                }
            });
        }
        else {
            var resultRect = this._scaleRect(this.cellRect);
            var oldScaledRect = this._scaleRect(lastRect);
            if (!WT.rectContains(oldScaledRect, this.cellRect)) {
                this.showCover = true;
                this.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, true);
            }
            this.fireEvent(WT.ReportBlock.EVENT_LOADING_STATUS, true);
            WT.net.getCellsByRange(o.sheetIndex, o.type, resultRect, o.sheet.element).then(function (data) {
                self.fetchedCells = data.cells;
            })['catch'](WT.emptyFn).then(function () {
                self.fireEvent(WT.ReportBlock.EVENT_LOADING_STATUS, false);
                if (self.showCover) {
                    self.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, false);
                    self.showCover = false;
                }
            });
        }
    },

    /**
     * 根据后台获取的cell列表生成单元格
     * @param cells
     * @param appendOnly {boolean} 是否仅添加而不删除原来的
     * @private
     */
    _applyCells: function (cells, appendOnly) {
        //已存在但新加载的列表中并没有的cell，该部分cell为移出范围应该销毁的
        var oldCellMap = $.extend({}, this.cellMap);

        //原来的列表和新加载的列表中都存在的cell（value对应的cell为新加载的列表中的cell的option）
        var existCellConfigArray = [];

        //原来的列表不存在但新加载的列表中存在的cell
        var newCellConfigArray = [];

        WT.each(cells, function (cell) {
            if (cell.forceRefresh === true) {
                //强制重新加载格子
                newCellConfigArray.push(cell);
            }
            else {
                delete oldCellMap[cell.id];
                if (this.cellMap[cell.id]) {
                    existCellConfigArray.push(cell);
                }
                else {
                    newCellConfigArray.push(cell);
                }
            }
        }, this);
        if (!appendOnly) {
            this._applyOldCells(oldCellMap);
            this._applyExistCells(existCellConfigArray);
        }
        this._applyNewCells(newCellConfigArray);
        if (!this.cellLoaded) {
            this.cellLoaded = true;
            this.fireEvent(WT.ReportBlock.EVENT_FIRST_LOADING_COMPLETE);
        }
    },

    _applyOldCells: function (oldCellMap) {
        WT.each(oldCellMap, function (cell, id) {
            delete this.cellMap[id];
            cell.umount();
            cell.destroy();
        }, this);
    },

    _applyExistCells: function (existCellConfigArray) {
        //已有项暂时不处理
    },

    _applyNewCells: function (newCellConfigArray) {
        var o = this.options, self = this;
        var dirtyMap = o.sheet.getDirtyCellMap();
        var invalidMap = o.sheet.getInvalidCellMap();
        WT.each(newCellConfigArray, function (cellConfig) {
            var id = cellConfig.id;
            var cell = new WT.Cell($.extend({
                sheetIndex: o.sheetIndex,
                isShowWidgets: o.isShowWidgets,
                paraProvider: WT.createParaProvider(o.sheet.sheetPara, o.sheetIndex),
                sheetConfigProvider: function () {
                    return o.sheet.sheetConfig
                },
                store: o.store,
                sheetContext: o.sheetContext
            }, WT.Cell.transformConfig(cellConfig)));
            cell.on(WT.Cell.EVENT_CELL_SELECT, function () {
                self._handleCellSelect(cell);
            });
            cell.on(WT.Cell.EVENT_SYNC_VALUE, function () {
                self.fireEvent(WT.Cell.EVENT_SYNC_VALUE, cell);
            });
            cell.on(WT.Cell.EVENT_ENTER_PRESSED, function () {
                o.sheet.selectDown();
            });
            cell.on(WT.Cell.EVENT_TAB_PRESSED, function () {
                o.sheet.selectNext();
            });
            this.cellMap[id] = cell;
            cell.mount(this.element);
            var cv = dirtyMap[id];
            cv && cell.setValue(cv.value, null, cv.originValue);
            var msg = invalidMap[id];
            msg && cell.setInvalid(msg);
        }, this);
    },

    _handleCellSelect: function (cell) {
        this.fireEvent(WT.Cell.EVENT_CELL_SELECT, cell);
    },

    _scaleRect: function (rect) {
        var multiple = this._const.scaleMultiple;
        var scaledWidth = rect.width * multiple;
        var scaledHeight = rect.height * multiple;
        var r = {
            left: rect.left - Math.floor(rect.width * (multiple - 1) / 2),
            top: rect.top - Math.floor(rect.height * (multiple - 1) / 2),
            width: scaledWidth,
            height: scaledHeight
        };
        if (r.left < 0) {
            r.width += r.left;
            r.left = 0;
        }
        if (r.top < 0) {
            r.height += r.top;
            r.top = 0;
        }
        return r;
    },

    /**
     * 请求显示/移除加载遮盖
     * @param show {Boolean}
     * @private
     */
    _requestLoadingCover: function (show) {
        this.fireEvent(WT.ReportBlock.EVENT_LOADING_COVER, show);
    },

    _scrollPageTo: function (left, top) {
        var lastRect = this.cellRect || {
            left: this.scrollLeft,
            top: this.scrollTop,
            width: this.size.width,
            height: this.size.height
        };
        var oldScaledRect = this._scaleRect(lastRect);
        var oldScrollLeft = this.scrollLeft;
        var oldScrollTop = this.scrollTop;

        var movementX = left - oldScrollLeft;
        var movementY = top - oldScrollTop;
        var isOutOfRect = false;
        if (movementX > 0 && left + this.size.width > oldScaledRect.width + oldScaledRect.left) {
            left = oldScaledRect.width - this.size.width + oldScaledRect.left;
            isOutOfRect = true;
        }
        if (movementX < 0 && left < oldScaledRect.left) {
            left = oldScaledRect.left;
            isOutOfRect = true;
        }
        if (movementY > 0 && top + this.size.height > oldScaledRect.height + oldScaledRect.top) {
            top = oldScaledRect.height - this.size.height + oldScaledRect.top;
            isOutOfRect = true;
        }
        if (movementY < 0 && top < oldScaledRect.top) {
            top = oldScaledRect.top;
            isOutOfRect = true;
        }
        this._notifyOutOfRectStatus(isOutOfRect);

        if (left !== this.scrollLeft) {
            this.element.scrollLeft(left);
            this.scrollLeft = left;
        }
        if (top !== this.scrollTop) {
            this.element.scrollTop(top);
            this.scrollTop = top;
        }
    },

    _notifyOutOfRectStatus: function (isOutOfRect) {
        var lastStatus = this.isScrollingOutOfRect;
        if (lastStatus !== isOutOfRect) {
            this.isScrollingOutOfRect = isOutOfRect;
            this.fireEvent(WT.ReportBlock.EVENT_SCROLL_OUT_OF_RECT_STATUS, isOutOfRect);
        }
    },

    applyFetchedData: function () {
        if (!this.fetchedCells) {
            return;
        }
        this._applyCells(this.fetchedCells);
        this._scrollPageTo(this.actualScrollLeft, this.actualScrollTop);
        this.fetchedCells = null;
    },

    /**
     * 开始/重新加载单元格数据
     */
    refreshCellData: function () {
        this._synchronizeCellData(true);
        this.loadStarted = true;
    },

    appendNewCells: function (cells) {
        this._applyCells(cells, true);
    },

    setBackground: function (bg) {
        if (!this.placeholder) {
            return;
        }
        if (bg) {
            this.placeholder.css(bg);
        } else {
            this.placeholder.css('background', null);
        }
    },

    /**
     * 设置单元格新值
     * @param newValue {object | array} {id, column, row, value} | [{id, column, row, value}]
     */
    applyNewValue: function (newValue) {
        var list = WT.isArray(newValue) ? newValue : [newValue];
        WT.each(list, function (cellValue) {
            var cell = this.cellMap[cellValue.id];
            if (cell) {
                cell.setValue(cellValue.value);
                cellValue.hy && cell.updateHyperlink(cellValue.hy);
            }
        }, this);
    },

    getCellMap: function () {
        return this.cellMap;
    },

	/**
	 * 设置大小
	 * @param x {Number} 左边距
	 * @param y {Number} 上边距
	 * @param width {Number} 宽度
	 * @param height {Number} 高度
	 */
	resize: function (x, y, width, height) {
        this.size = {
            left: x,
            top: y,
            width: width,
            height: height
        };
        if (this.options.type === 'north') {
            this.element.css({
                left: x,
                top: y,
                width: width,
                height: height + 3
            });
        }
        else {
            this.element.css({
                left: x,
                top: y,
                width: width,
                height: height
            });
        }
        this.placeholder.css({
            left: 0,
            top: 0,
            width: this.options.contentWidth,
            height: this.options.contentHeight
        });
        if (!this.loadStarted) {
            return;
        }
        this._handleRectChange();
	},

    /**
     * 代码控制滚动内容
     * @param left {Number} 水平偏移
     * @param top {Number} 垂直偏移
     * @param type {'wheel' | 'drag' | 'click'} 触发滚动的类型
     */
	scrollTo: function (left, top, type) {
        if (!WT.isNumber(left)) {
            left = this.scrollLeft;
        }
        this.actualScrollLeft = left;
        if (!WT.isNumber(top)) {
            top = this.scrollTop;
        }
        this.actualScrollTop = top;
	    this._scrollPageTo(left, top);
        if (!this.loadStarted) {
            return;
        }
        this._handleRectChange(type);
	},

    onScrollBarDragStart: function (isVertical) {
        // empty
    },

    onScrollBarDragStop: function (isVertical) {
        if (this.needToSyncAfterDragStop) {
            this.needToSyncAfterDragStop = false;
            this._synchronizeCellData();
        }
    },

    getScrollRect: function () {
        return {
            left: this.scrollLeft,
            top: this.scrollTop,
            width: this.size.width,
            height: this.size.height
        }
    },

    getContentWidth: function () {
        return this.options.contentWidth;
    },

    getContentHeight: function () {
        return this.options.contentHeight;
    },

    /**
     * 锁住窗口滚动
     * @param lock {Boolean} 是否锁住
     */
    lockScroll: function (lock) {
        // empty
    },

    isLoadCompleted: function () {
        return this.cellLoaded;
    },

    destroy: function () {
        this.purgeListeners();
        this.placeholder.off('click');
        this.element.off('scroll');
        this.element.empty();
        this.element.remove();
        this.element = null;
    }

});

WT.ReportBlock.EVENT_REQUEST_RESIZE = 'event-request-resize';
WT.ReportBlock.EVENT_LOADING_COVER = 'EVENT_LOADING_COVER';
WT.ReportBlock.EVENT_LOADING_STATUS = 'EVENT_LOADING_STATUS';
WT.ReportBlock.EVENT_FIRST_LOADING_COMPLETE = 'EVENT_FIRST_LOADING_COMPLETE';
WT.ReportBlock.EVENT_CELL_UNSELECT = 'EVENT_CELL_UNSELECT';
WT.ReportBlock.EVENT_SCROLL_OUT_OF_RECT_STATUS = 'EVENT_SCROLL_OUT_OF_RECT_STATUS';
WT.ReportBlock.EVENT_CELLS_RELOAD = 'EVENT_CELLS_RELOAD';