/**
 * 新填报下页面根容器
 * @extends FR.OB
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.warnWhenLeave 未提交离开提示
 * @cfg {Boolean} options.isAutoStash 自动暂存
 * @cfg {Boolean} options.isShowWidgets 直接显示控件
 * @cfg {Boolean} options.isAlignLeft 是否靠左显示
 * @cfg {String} options.selectedLineColor 当前编辑行颜色
 * @cfg {Object} options.topToolBarConfig 顶部工具栏，见WT.ToolBar
 * @cfg {Object} options.bottomToolBarConfig 底部工具栏，见WT.ToolBar
 * @cfg {Object} options.parameterPanelConfig 参数面板，见WT.ParameterPanel
 * @cfg {Object} options.sheetContainerConfig 表单容器，见WT.SheetContainer
 */
WT.ReportPage = FR.extend(FR.OB, {

    _defaultConfig: function () {
        return $.extend(WT.ReportPage.superclass._defaultConfig.apply(), {
            warnWhenLeave: true,
            isAutoStash: false,
            isShowWidgets: false,
            isAlignLeft: true,
            selectedLineColor: null,
            topToolBarConfig: null,
            bottomToolBarConfig: null,
            parameterPanelConfig: null,
            sheetContainerConfig: null,
            background: null
        });
    },

    _init: function () {
        WT.ReportPage.superclass._init.apply(this, arguments);
        this._initVariable();
        this._initView();
        this._initEvents();
        WT.nextTick(function () {
            this.currentSessionID = FR.SessionMgr.getSessionID();
        }, this);
    },

    _initVariable: function () {
        this.selectedIndex = 0;
        this.selectedCellLocation = null;
        this.isCalculating = false;
        this.calculatingCount = 0;

        this.globalConfig = {
            editOnClick: false,
            editOnMove: false,
            fireOnChange: false
        };

        this.stashDebounced = WT.debounce(this.stash, 500);
        this.store = new WT.Store();
    },

    /**
     * 初始化视图，子组件
     */
    _initView: function () {
        this.element = $('<div class="report-page wt-absolute-layout"></div>');
        var toolBarItems = [];
        if (this.options.topToolBarConfig) {
            this.options.topToolBarConfig.store = this.store;
            this.topToolBar = new WT.ToolBar(this.options.topToolBarConfig);
            this.element.append(this.topToolBar.getRootElement());
            WT.each(this.topToolBar.items, function (item) {
                toolBarItems.push(item);
            });
        }
        if (this.options.parameterPanelConfig) {
            this.options.parameterPanelConfig.store = this.store;
            this.parameterPanel = new WT.ParameterPanel(this.options.parameterPanelConfig);
            this.element.append(this.parameterPanel.getRootElement());
            window.form = this.parameterPanel;
        }
        if (this.options.sheetContainerConfig) {
            this.options.sheetContainerConfig.store = this.store;
            this.sheetContainer = new WT.SheetContainer($.extend(this.options.sheetContainerConfig, {
                report: this
            }));
            this.element.append(this.sheetContainer.getRootElement());

            this.curLGP = new WT.WLGP({
                report: this
            });
        }
        if (this.options.bottomToolBarConfig) {
            this.options.bottomToolBarConfig.store = this.store;
            this.bottomToolBar = new WT.ToolBar(this.options.bottomToolBarConfig);
            this.element.append(this.bottomToolBar.getRootElement());
            WT.each(this.bottomToolBar.items, function (item) {
                toolBarItems.push(item);
            });
        }
        this.element.appendTo($('body'));
        if (this.options.background) {
            this.element.css(this.options.background);
        }

        this.pasteHelperText = $('<textarea class="wt-paste-helper-textarea"></textarea>');
        this.pasteHelperText.keyup(function () {
            this.blur();
        });
        this.pasteHelperText.appendTo(this.element);

        this.toolbar = {
            options: {
                items: toolBarItems
            }
        };
    },

    /**
     * 初始化事件
     */
    _initEvents: function () {
        var self = this;
        //页面缩放
        var $window = $(window);
        $window.resize(WT.debounce(function () {
            var width = $window.width();
            var height = $window.height();
            self._resize(width, height);
            self.fireEvent(FR.Events.RESIZE, {
                width: width,
                height: height
            });
        }, 300));
        this._resize($window.width(), $window.height());

        if (this.sheetContainer) {
            this.sheetContainer.on(FR.Events.STARTLOAD, function (sheetIndex) {
                self._handleReportLoadStart(sheetIndex);
            });
            this.sheetContainer.on(FR.Events.AFTERLOAD, function (sheetIndex) {
                self._handleReportLoadEnd(sheetIndex);
            });
            this.sheetContainer.on(WT.Cell.EVENT_CELL_SELECT, function (cell) {
                self._handleCellSelect(cell);
            });
            this.sheetContainer.on(WT.SheetContainer.EVENT_SHEET_CHANGE, function (sheetIndex) {
                self.selectedIndex = sheetIndex;
            });
            this.sheetContainer.on(WT.SheetContainer.EVENT_CALCULATING_CELL, function (isCalculating) {
                self.isCalculating = isCalculating;
                if (!isCalculating) {
                    self.fireEvent(WT.ReportPage.EVENT_CALCULATING_FINISH);
                }
            });
        }

        //参数面板
        if (this.parameterPanel) {
            this.parameterPanel.on(WT.ParameterPanel.EVENT_COLLAPSED, this._resizeChildComp.bind(this));
            this.parameterPanel.on(WT.ParameterPanel.EVENT_EXPANDED, this._resizeChildComp.bind(this));
            this.parameterPanel.on(WT.ParameterPanel.EVENT_CALCULATING_WIDGET, function (isCalculating) {
                if (isCalculating) {
                    self.isCalculating = true;
                    self.calculatingCount += 1;
                }
                if (!isCalculating) {
                    self.calculatingCount -= 1;
                }
                if (self.calculatingCount <= 0) {
                    self.isCalculating = false;
                    self.fireEvent(WT.ReportPage.EVENT_CALCULATING_FINISH);
                }
            });

            if (this.options.parameterPanelConfig.donotShowReportBeforeSubmit === false) {
                //点击查询前直接显示报表内容
                WT.nextTick(this.submitParameter, this);
            }
        } else {
            //没有参数面板，直接触发加载报表
            WT.nextTick(this.loadReportData, this);
        }

        //页面转向时提示,不支持opera
        window.onbeforeunload = function () {
            return self._beforeUnloadCheck();
        };
    },

    /**
     * 缩放，首次加载和浏览器窗口变化都会调用
     * @param width {Number} 页面宽度
     * @param height {Number} 页面高度
     */
    _resize: function (width, height) {
        var size = {
            top: 0,
            left: 0,
            width: width,
            height: height
        };
        this.windowSize = size;
        this.element.css(size);
        this._resizeChildComp();
    },

    /**
     * 缩放子组件，首次加载，页面缩放和参数面板收起/展开时会调用
     */
    _resizeChildComp: function () {
        var width = this.windowSize.width;
        var height = this.windowSize.height;
        var offset = 0, offsetBottom = 0;
        //先从子组件的属性中获取高度，再计算出子组件应该摆放的位置，设置回去。ToolBar和ParameterPanel都用这种方法
        if (this.topToolBar) {
            var topToolBarHeight = this.topToolBar.getHeight();
            this.topToolBar.resize(0, offset, width, topToolBarHeight);
            offset += topToolBarHeight;
        }
        if (this.parameterPanel) {
            var parameterPanelHeight = this.parameterPanel.getHeight();
            this.parameterPanel.resize(0, offset, width, parameterPanelHeight);
            offset += parameterPanelHeight;
        }
        if (this.bottomToolBar) {
            var bottomToolBarHeight = this.bottomToolBar.getHeight();
            this.bottomToolBar.resize(0, height - bottomToolBarHeight, width, bottomToolBarHeight);
            offsetBottom += bottomToolBarHeight;
        }
        if (this.sheetContainer) {
            this.sheetContainer.resize(0, offset, width, height - offset - offsetBottom);
        }
    },

    _beforeUnloadCheck: function () {
        this._stopEdit();
        if (this.store.isReportModified()) {
            if (this.options.warnWhenLeave !== false) {
                return FR.i18nText("Fine-Engine_Unload_Check_Msg") + "!";
            }
        }
    },

    _handleReportLoadStart: function (sheetIndex) {
        this.fireEvent(FR.Events.STARTLOAD, sheetIndex);
        if (this.topToolBar) {
            this.topToolBar.setEnable(false);
        }
        if (this.bottomToolBar) {
            this.bottomToolBar.setEnable(false);
        }
    },

    _handleReportLoadEnd: function (sheetIndex) {
        if (this.topToolBar) {
            this.topToolBar.setEnable(true);
        }
        if (this.bottomToolBar) {
            this.bottomToolBar.setEnable(true);
        }
        var self = this;
        var keyDownFn = this._handleKeyDown.bind(this);

        WT.Keys.reg(keyDownFn);

        $(document).mousedown(function (e) {
            if (FR.isAncestor(self.sheetContainer.element[0], e.target)) {
                WT.Keys.reg(keyDownFn);
            } else {
                WT.Keys.unreg(keyDownFn);
            }
        });
        this.fireEvent(FR.Events.AFTERLOAD, sheetIndex);
    },

    _handleCellSelect: function (cell) {
        this.fireEvent(WT.Cell.EVENT_CELL_SELECT, this.curLGP._getTDCellByCell(cell));
    },

    /**
     * 处理校验返回数据
     * @param resultArray
     * @return {{info: {}, errors: Array, success: boolean}}
     * @private
     */
    _handleVerifyResult: function (resultArray) {
        var resultJSON = [];
        var verifyInfo = {};
        WT.each(resultArray, function (item) {
            if (item.fr_verifyinfo) {
                verifyInfo = item.fr_verifyinfo;
            } else {
                resultJSON.push(item);
            }
        });
        return {
            info: verifyInfo,
            errors: resultJSON,
            success: resultJSON.length === 0
        }
    },

    _reloadSheet: function (sheetIndex) {
        if (WT.isNumber(sheetIndex)) {
            this.sheetContainer.reloadSheet(sheetIndex);
        } else {
            this.sheetContainer.reloadSheetAll();
        }
    },
    _clearDirty: function (sheetIndex) {
        if (WT.isNumber(sheetIndex)) {
            this.sheetContainer.clearDirtyCells(sheetIndex);
        } else {
            this.sheetContainer.clearDirtyCellsAll();
            this._afterContentSave();
        }
    },

    _handleKeyDown: function (e) {
        var sheet = this.getCurrentSheet();
        if (!sheet.isLoaded()) {
            return;
        }
        switch (e.keyCode) {
            case 38:   //up
                e.preventDefault();
                sheet.selectUp();
                break;
            case 40:   //down
                e.preventDefault();
                sheet.selectDown();
                break;
            case 37:   //left
                e.preventDefault();
                sheet.selectPrev();
                break;
            case 39:   //right
                e.preventDefault();
                sheet.selectNext();
                break;
            case 13:   //enter
                e.preventDefault();
                if (e.shiftKey) {
                    sheet.selectUp();
                } else {
                    sheet.selectDown();
                }
                break;
            case 9:    //tab
                e.preventDefault();
                if (e.shiftKey) {
                    sheet.selectPrev();
                } else {
                    sheet.selectNext();
                }
                break;
            case 46:   //delete
            case 8:    //backspace
                e.preventDefault();
                var cell = sheet.getSelectedCell();
                if (cell && cell.options.isEditor) {
                    sheet.clearCellValue();
                }
                break;
            default:
                if (e.ctrlKey || e.metaKey) {
                    if (e.keyCode === 67) {
                        sheet.editCell();
                    } else if (e.keyCode === 86) {
                        var el;
                        if (FR.Browser.isIE()) {
                            el = e.srcElement;
                        } else {
                            el = e.target;
                        }
                        var tag = el.tagName;
                        if (tag !== "INPUT" && tag !== "TEXTAREA") {
                            var cp = this.pasteHelperText[0];
                            var self = this;
                            cp.focus();
                            cp.select();
                            setTimeout(function () {
                                sheet.handlePaste(self.pasteHelperText.val());
                            }, 50);
                        }
                    }
                    break;
                }
                if (!e.ctrlKey
                    && !e.metaKey
                    && !e.altKey) {
                    if (WT.isPrintableKey(e.keyCode)) {
                        sheet.editCell();
                    } else if (e.keyCode === 32) {
                        e.preventDefault();
                        sheet.editCell();
                    }
                }
                break;
        }
    },

    /**
     * 处理校验错误，提示错误信息等
     * @private
     */
    _applyVerifyError: function (errors, btn) {
        if (!errors || !WT.isArray(errors)) {
            return;
        }
        var self = this;
        var errorList = [];
        var errorList4GetCellId = [];
        WT.each(errors, function (error) {
            WT.each(error.columnrows, function (cr) {
                errorList.push({
                    sheetIndex: error.reportIndex,
                    columnRow: cr,
                    msg: error.message
                });
                errorList4GetCellId.push({
                    sheetIndex: error.reportIndex,
                    columnRow: cr
                });
            });
        });
        WT.net.getCellID(errorList4GetCellId, true).then(function (result) {
            return result.list;
        }).then(function (list) {
            if (list !== undefined) {
                if (list.length !== errorList.length) {
                    return WT.createPromiseError();
                }
                var idMap = {}, rectMap = {}, blockTypeMap = {};
                WT.each(list, function (l, i) {
                    idMap[errorList[i].columnRow] = l.id;
                    rectMap[l.id] = l.rect;
                    errorList[i].id = l.id;
                    errorList[i].rect = l.rect;
                    blockTypeMap[l.id] = l.blockType;
                });

                var errorInfoMap = WT.reduce(errorList, function (prev, error) {
                    var sheetErrorList = prev[error.sheetIndex];
                    if (!sheetErrorList) {
                        sheetErrorList = [];
                        prev[error.sheetIndex] = sheetErrorList;
                    }
                    sheetErrorList.push({
                        id: error.id,
                        msg: error.msg
                    });
                    return prev;
                }, {});
                //按照sheetIndex划分的单元格校验错误信息，[{sheetIndex, cells: [{id, msg}]}]
                var errorInfoList = [];
                WT.each(errorInfoMap, function (error, sheetIndex) {
                    errorInfoList.push({
                        sheetIndex: parseInt(sheetIndex),
                        cells: error
                    });
                });
                self.sheetContainer.setInvalidInfo(errorInfoList);

                //按照错误信息划分的校验错误信息，[msg, sheetIndex, cellRectList: [id, blockType, name, rect:{left, top, width, height}]]
            }
            var errorInfoList4Popup = WT.map(errors, function (error) {
                var cellRectList = WT.map(error.columnrows, function (cr) {
                    var id = idMap[cr];
                    return {
                        id: id,
                        rect: rectMap[id],
                        blockType: blockTypeMap[id],
                        name: cr
                    };
                });
                return {
                    cellRectList: cellRectList,
                    sheetIndex: error.reportIndex,
                    msg: error.message
                }
            });

            self._showVerifyErrorPopup(errorInfoList4Popup, btn);

            self._setVerifyBadge(btn, errorInfoList4Popup.length);

        })['catch'](WT.emptyFn);
    },

    _clearVerifyError: function () {
        this.sheetContainer.clearInvalidInfo();
        this._hideVerifyErrorPopup();
        this._clearVerifyBadge();
    },

    _getDirtyCellList: function (sheetIndex) {
        return [{
            sheetIndex: sheetIndex === undefined ? this.selectedIndex : sheetIndex,
            cells: this.sheetContainer.getDirtyCells(this.selectedIndex)
        }];
    },

    _getDirtyCellListAll: function () {
        return WT.map(this.sheetContainer.getDirtyCellsAll(), function (dirtyCellList, idx) {
            return {
                sheetIndex: idx,
                cells: dirtyCellList
            }
        });
    },

    /**
     * 停止编辑当前的格子
     * @return {*}
     * @private
     */
    _stopEdit: function (isForceSyncValue) {
        var cell = this.getSelectedCell();
        if (cell) {
            var r = cell.stopEdit(isForceSyncValue);
            cell.unSelect();
            return r;
        }
        return true;
    },

    _showVerifyErrorPopup: function (errorInfoList, btn) {
        var self = this;
        this._hideVerifyErrorPopup();
        var errors = WT.map(errorInfoList, function (error) {
            var position = WT.reduce(error.cellRectList, function (prev, next) {
                if (prev === '') {
                    return next.name;
                } else {
                    return prev + ' ' + next.name;
                }
            }, '');
            return {
                sheetIndex: error.sheetIndex,
                msg: error.msg,
                position: position
            }
        });
        this.verifyPopup = new WT.VerifyPopup({
            errors: errors
        });
        self._fixVerifyPopup(btn);
        this.verifyPopup.on(WT.VerifyPopup.EVENT_CLOSE, function () {
            self._hideVerifyErrorPopup();
        });
        var keyDownFn = this._handleKeyDown.bind(this);
        this.verifyPopup.on(WT.VerifyPopup.EVENT_SELECT, function (idx) {
            var error = errorInfoList[idx];
            var cells = error.cellRectList;
            if (!cells || cells.length === 0) {
                return;
            }
            var cell = cells[0];
            self.sheetContainer.locateAndSelectCell(error.sheetIndex, cell);
            WT.Keys.reg(keyDownFn);//点击校验
        });

    },

    _fixVerifyPopup: function (btn) {
        if (btn && btn.element) {
            this.verifyPopup.element.css('left', btn.element.offset().left);
        }
    },

    _hideVerifyErrorPopup: function () {
        this.verifyPopup && this.verifyPopup.destroy();
        this.verifyPopup = null;
    },

    _setVerifyBadge: function (btn, count) {
        if (btn && WT.isFunction(btn.setBadge)) {
            btn.setBadge(count);
            this.verifyBtn = btn;
        }
    },

    _clearVerifyBadge: function (btn) {
        this.verifyBtn && this.verifyBtn.setBadge(0);
        if (btn && WT.isFunction(btn.setBadge)) {
            btn.setBadge(0);
        }
    },

    _afterContentSave: function () {
        this.store.setReportModified(false);
    },

    clearReport: function () {
        this.sheetContainer.clearSheetData();
    },

    isDirtyPage: function () {
        return this.store.isReportModified();
    },

    /**
     * 开始加载报表
     */
    loadReportData: function () {
        this.sheetContainer.loadSheetData();
    },

    /**
     * 根据位置获取单元格
     * @param location 包含{sheetIndex, column, row}
     */
    getCell: function (location) {
        return this.sheetContainer.getCell(location);
    },

    /**
     * 获取当前选中的单元格
     */
    getSelectedCell: function () {
        var sheetIndex = this.getSheetIndex();
        return this.sheetContainer.getSelectedCell(sheetIndex);
    },

    /**
     * 获取当前选中sheetIndex
     * @return {number|*}
     */
    getSheetIndex: function () {
        return this.selectedIndex;
    },

    getCurrentSheet: function () {
        return this.sheetContainer.getActiveSheet();
    },

    /**
     * 获取当前sheet的根元素
     * @return {*|null}
     */
    getSheetElement: function () {
        var sheetIndex = this.getSheetIndex();
        return this.sheetContainer.getSheetElement(sheetIndex);
    },

    getSheetsInfo: function () {
        return this.sheetContainer.getSheetsInfo();
    },

    getSheet: function (sheetIndex) {
        return this.sheetContainer.sheets[sheetIndex];
    },

    /**
     * 重新加载当前sheet
     */
    reloadSheet: function () {
        var sheetIndex = this.getSheetIndex();
        this.sheetContainer.reloadSheet(sheetIndex);
    },

    /**
     * 重新加载当前sheet
     */
    reloadSheetAfterAppendOrDelete: function (appendLocation) {
        var sheetIndex = this.getSheetIndex();
        this.sheetContainer.clearDirtyCells(sheetIndex);
        if (appendLocation) {
            this.sheetContainer.reloadSheetAfterAppendOrDelete(sheetIndex, appendLocation);
        } else {
            this.sheetContainer.reloadSheet(sheetIndex);
        }
    },

    loadContentPane: function () {
        this.sheetContainer.loadSheetData();
    },

    /**
     * 等待当前控件同步值
     * @return {*|Promise.<T>}
     */
    waitForCalculation: function () {
        if (this.isCalculating) {
            var self = this;
            return WT.createPromise(function (resolve) {
                self.once(WT.ReportPage.EVENT_CALCULATING_FINISH, resolve);
            });
        } else {
            return WT.createPromiseResult();
        }
    },

    stopEditAndWaitCal: function (isForceSyncValue) {
        var self = this;
        return WT.createPromiseResult(true).then(function () {
            if (!self._stopEdit(isForceSyncValue) && isForceSyncValue !== true) {
                return false;
            }
            return self.waitForCalculation().then(function () {
                return true;
            });
        });
    },

    validateWidgets: function (resultJSON, editorsConfig) {
        for (var i = 0; i < editorsConfig.length; i++) {
            var sheetEditorConfig = editorsConfig[i];
            var sheetIndex = sheetEditorConfig.sheetIndex;
            var configs = sheetEditorConfig.config;
            var dirtyCellMap = this.sheetContainer.getDirtyCellMap(sheetIndex) || {};
            for (var j = 0; j < configs.length; j++) {
                var widget = configs[j];
                var location = widget.cr;
                var value = widget.value;
                if (dirtyCellMap[widget.id]) {
                    value = dirtyCellMap[widget.id].value;
                }
                if (value === undefined || value === null || value === "null") {
                    value = "";
                }
                if (widget.allowBlank === false && value === "") {
                    var errorMsg = widget.errorMsg || FR.i18nText("Fine-Engine_Not_NULL");
                    resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                    continue;
                } else if (widget.allowBlank !== false && value === "") {
                    continue;
                }
                if (widget.allowBlank !== false && value === "") {
                    continue;
                }
                if (widget.regex) {
                    if (!this.validRegexWidget(widget, location, resultJSON, value, sheetIndex)) {
                        continue;
                    }
                }
                if (widget.type === "number") {
                    var num = value, errorMsg = '', verify = true;
                    if (value != '' && isNaN(num)) {
                        var errorMsg = widget.regErrorMsg || FR.i18nText("Fine-Engine_Err_The_Value_Must_Be_Number");
                        resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                        continue;
                    }
                    num = isNaN(num) ? '' : num;
                    var numString = '' + num;
                    if (numString.indexOf(".") > 0) {
                        if (widget.allowDecimals === false) {
                            errorMsg = widget.regErrorMsg || FR.i18nText("Fine-Engine_Err_The_Value_Must_Be_Integer");
                            resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                            continue;
                        } else if (widget.allowDecimals !== false) {
                            // 判断小数位数 去掉负号
                            if (numString.startWith('-')) {
                                numString = numString.substring(1);
                            }
                            // shoc:科学计数法处理下
                            if (numString.indexOf('E') !== -1) {
                                var es = parseInt(numString.substring(numString.lastIndexOf('E') + 1, numString.length));
                                if (numString.indexOf('E') - 1 - numString.indexOf(".") > widget.maxDecLength + es) {
                                    errorMsg = widget.regErrorMsg || FR.i18nText("Fine-Engine_Decimal_Number_Out");
                                    resultJSON.push({
                                        message: errorMsg,
                                        columnrows: [location],
                                        reportIndex: sheetIndex
                                    });
                                    continue;
                                }
                            } else {
                                if (numString.length - numString.indexOf(".") > widget.maxDecLength + 1) {
                                    errorMsg = widget.regErrorMsg || FR.i18nText("Fine-Engine_Decimal_Number_Out");
                                    resultJSON.push({
                                        message: errorMsg,
                                        columnrows: [location],
                                        reportIndex: sheetIndex
                                    });
                                    continue;
                                }
                            }
                        }
                    }
                    if (widget.allowNegative === false && num < 0) {
                        errorMsg = widget.regErrorMsg || FR.i18nText("FIne-Engine_Err_The_Value_Cannot_Be_Negative");
                        resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                        continue;
                    }
                    if (widget.minValue != null && num < widget.minValue) {
                        errorMsg = widget.regErrorMsg || FR.i18nText("Fine-Engine_Err_The_Number_Is_Less_Than_The_Minimum_Value") + widget.minValue;
                        resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                        continue;
                    }
                    if (widget.maxValue != null && num > widget.maxValue) {
                        errorMsg = widget.regErrorMsg || FR.i18nText("Fine-Engine_Err_The_Number_Is_Larger_Than_The_Maximum_Value") + widget.maxValue;
                        resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                    }
                }
            }
        }
    },

    validRegexWidget: function (widget, location, resultJSON, value, sheetIndex) {
        var errorMsg = widget.regErrorMsg || FR.i18nText("Fine-Engine_Error_Input_Value");
        for (var indexOfPlugin = 0, lengthOfPlugin = FR.BaseEditor.PluginRegex.length; indexOfPlugin < lengthOfPlugin; indexOfPlugin++) {
            var obj = FR.BaseEditor.PluginRegex[indexOfPlugin];
            if (obj.text === widget.regex) {
                if (obj.regex(value)) {
                    return true;
                } else {
                    resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                    return false;
                }
            }
        }
        if (typeof widget.regex == 'string') {
            var regex = new RegExp(widget.regex);
        }
        if (!regex.test(value)) {
            /* 正则表达式是判断长度的 换行符算两个字符*/
            if (!(regex.source.indexOf(".{") > -1 && regex.test((value + '').replace(/\n/g, "aa")))) {
                resultJSON.push({message: errorMsg, columnrows: [location], reportIndex: sheetIndex});
                return false;
            }
        }
        return true;
    },

    trigAutoStash: function () {
        if (this.options.isAutoStash) {
            this.stashDebounced();
        }
    },

    /**
     * 填报入库
     * 填报入库第一个sheet，成功的话alert成功，失败就alert失败：
     *
     *     @example
     *     contentPane.writeReport(0, null, function(){alert("成功");}, function(){alert("失败");});
     *
     * @param reportIndex
     * @param submitButton
     * @param successFn
     * @param failFn
     * @return {boolean}
     */
    writeReport: function (reportIndex, submitButton, successFn, failFn) {
        this._clearVerifyError();
        if (!this._stopEdit(true)) {
            return false;
        }
        var sheetIndex = WT.isNumber(reportIndex) ? reportIndex : null;
        var self = this;
        this.waitForCalculation().then(function () {
            self.doVerifyAndWriteReport(failFn, false, sheetIndex, submitButton, successFn, failFn);
        });
        return true;
    },

    /**
     * 校验并提交修改数据,该函数在服务器端写死了调用
     *
     * 绑定成功失败事件可以用on动态绑定
     * 但是有时候客户并不引用我们的js这个时候就传入方法
     *
     * @param isAllSheet
     * @param submitButton
     * @param successFn
     * @param failFn
     * @returns {boolean}  提交成功返回true，否则返回false
     */
    verifyAndWriteReport: function (isAllSheet, submitButton, successFn, failFn) {
        this._clearVerifyError();
        this._stopEdit(true);
        var sheetIndex = isAllSheet ? null : this.getSheetIndex();
        var self = this;
        this.waitForCalculation().then(function () {
            self.doVerifyAndWriteReport(true, false, sheetIndex, submitButton, successFn, failFn);
        });
        return true;
    },

    /**
     * 忽略校验出错信息，直接提交
     * 提交按钮勾选校验失败仍然提交执行的方法
     * 705前是只提交当前sheet 改为提交全部sheet 48285
     *
     * @param isAllSheet 是否提交全部sheet
     */
    writeReportIgnoreVerify: function (isAllSheet) {
        this._clearVerifyError();
        //忽略控件校验出错信息,和老填报保持一致
        this._stopEdit(true);
        var sheetIndex = isAllSheet ? null : this.getSheetIndex();
        var self = this;
        this.waitForCalculation().then(function () {
            self.doVerifyAndWriteReport(true, true, sheetIndex);
        });
        return true;
    },

    doVerifyAndWriteReport: function (needVerify, ignoreVerifyResult, sheetIndex, submitButton, successFn, failFn) {
        var self = this;
        if (this.fireEvent(FR.Events.BVW, this) === false) {
            return false;
        }
        WT.setWidgetEnable(submitButton, false);
        var o = {
            overlay_immediately: true,
            text: FR.i18nText("Fine-Engine_Cal_Calculating") + "...."
        };
        FR.showLoadingDialog(o);
        var cellList = !WT.isNumber(sheetIndex) ? this._getDirtyCellListAll() : this._getDirtyCellList(sheetIndex);
        //step1: 校验
        WT.createPromiseResult().then(function () {
            if (needVerify) {
                return WT.net.writeVerify(cellList, sheetIndex).then(function (result) {
                    var list = result.verifyList;
                    var verifyResult = self._handleVerifyResult(list);
                    self._mergeWidgetsVerifyResult(verifyResult, result);
                    if (self.fireEvent(FR.Events.AV, verifyResult.info) === false) {
                        return WT.createPromiseError();
                    }
                    if (!verifyResult.success || verifyResult.errors.length > 0) {
                        if (ignoreVerifyResult) {
                            self._applyVerifyError(verifyResult.errors, submitButton);
                        } else {
                            return WT.createPromiseError({
                                payload: verifyResult,
                                type: 'verify'
                            });
                        }
                    }

                    if (self.fireEvent(FR.Events.BW, this) === false) {
                        return WT.createPromiseError();
                    }
                    return 'verified';
                });
            }
        }).then(function (status) {
            //step2: 提交
            //如果之前发生了校验动作，校验的时候提交过的数据不需要再次提交了
            var cellList4Submit = status === 'verified' ? WT.map(cellList, function (item) {
                var it = $.extend({}, item);
                it.cells = [];
                return it;
            }) : cellList;
            return WT.net.submitReport(cellList4Submit, sheetIndex);
        }).then(function (list) {
            self._clearDirty(sheetIndex);
            return list;
        }).then(function (list) {

            var submitInfo, builtinFail;
            WT.each(list, function (item) {
                if (item.fr_submitinfo) {
                    submitInfo = item.fr_submitinfo;
                }
                if (item.builtinFail) {
                    builtinFail = item.builtinFail;
                }
            });
            if (self.fireEvent(FR.Events.AVW, self, submitInfo) === false) {
                return WT.createPromiseError();
            }
            if (self.fireEvent(FR.Events.AW, submitInfo) === false) {
                return WT.createPromiseError();
            }
            var success = submitInfo.success;
            if (success !== true && success !== false) {
                // carl:这里表示没有填报属性设置，提示用户即可
                FR.Msg.toast(FR.i18nText("Fine-Engine_Report") + FR.i18nText("Fine-Engine_Report_Write_Attributes_Msg"));
                if (WT.isFunction(successFn)) {
                    successFn();
                }
                return WT.createPromiseError();
            } else if (success === true) {
                //step3: 强制清空暂存数据
                self.clear(undefined, false);//提交成功强制清空暂存数据
                FR.Msg.toast(FR.i18nText("Fine-Engine_Report_Successfully"));

                self.fireEvent(FR.Events.WS);
                if (WT.isFunction(successFn)) {
                    successFn();
                }
            } else if (success === false) {
                return WT.createPromiseError({
                    payload: {
                        submitInfo: submitInfo,
                        builtinFail: builtinFail
                    },
                    type: 'submit'
                });
            }
        }).then(function () {
            return WT.net.refreshReport(sheetIndex);
        })['catch'](function (err) {
            if (!err) {
                return;
            }
            if (err.type === 'verify') {
                //处理校验错误
                var verifyResult = err.payload;
                self._applyVerifyError(verifyResult.errors, submitButton);
                return;
            }
            if (err.type === 'submit') {
                //处理提交错误
                var submitInfo = err.payload.submitInfo;
                var builtinFail = err.payload.builtinFail;
                if (submitInfo.failedcell) {
                    self._applyVerifyError(submitInfo.failedcell, submitButton)
                }
                FR.Msg.toast(builtinFail ? FR.i18nText("Fine-Engine_Failed") : submitInfo.failinfo);
                self.fireEvent(FR.Events.WF);
                if (WT.isFunction(failFn)) {
                    failFn();
                }
                return;
            }
            WT.toastError(err);
        }).then(function () {
            WT.setWidgetEnable(submitButton, true);
            FR.hideLoadingDialog();
        });
        return true;
    },

    /**
     * 数据校验
     *
     * @param btn 校验触发按钮
     * @param sheetIndex
     */
    verifyReport: function (btn, sheetIndex) {
        this._clearVerifyError();
        this._stopEdit();
        var self = this;
        if (this.fireEvent(FR.Events.BVW, this) === false) {
            return false;
        }
        WT.setWidgetEnable(btn, false);
        this.waitForCalculation().then(function () {
            var cellList = !WT.isNumber(sheetIndex) ? self._getDirtyCellListAll() : self._getDirtyCellList(sheetIndex);
            WT.net.writeVerify(cellList, sheetIndex).then(function (result) {
                var list = result.verifyList;
                var verifyResult = self._handleVerifyResult(list);
                self._mergeWidgetsVerifyResult(verifyResult, result);
                if (self.fireEvent(FR.Events.AV, verifyResult.info) === false) {
                    return WT.createPromiseError();
                }
                if (!verifyResult.success || verifyResult.errors.length > 0) {
                    return WT.createPromiseError({
                        payload: verifyResult,
                        type: 'verify'
                    });
                }
            }).then(function () {
                FR.Msg.toast(FR.i18nText("Fine-Engine_Verify_Verify_Success"));
            })['catch'](function (err) {
                if (!err) {
                    return;
                }
                if (err && err.type === 'verify') {
                    self._applyVerifyError(err.payload.errors, btn);
                    return;
                }
                WT.toastError(err);
            }).then(function () {
                WT.setWidgetEnable(btn, true);
            });
        });
        return true;
    },
    _mergeWidgetsVerifyResult: function (verifyResult, result) {
        this.validateWidgets(verifyResult.errors, result.editorConfigList);
        //如果填报属性里没有设置校验而前端控件校验失败 这时候应该返回失败
        if (verifyResult.errors && verifyResult.errors.length > 0) {
            if (verifyResult.info) {
                verifyResult.info.success = false;
                // verifyResult 同时包含前后端校验结果，且verifyInfo.info.info有可能是字符串，先把他变成空数组
                // 新填报这边是 verifyResult.info.info 在记录
                verifyResult.info.info = [];
                // verifyResult 是一个数组，其每一个元素有两个属性columnrows和message，其中columnrows也是一个数组
                for (var i = 0; i < verifyResult.errors.length; i++) {
                    var columnRows = verifyResult.errors[i].columnrows;
                    var columnRowsStr = '';
                    for (var j = 0; j < columnRows.length; j++) {
                        columnRowsStr += columnRows[j]
                    }
                    verifyResult.info.info.push(columnRowsStr + ':' + verifyResult.errors[i].message);
                }
            }
        }
    },

    /**
     * flash打印
     */
    flashPrint: function () {
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
            }).then(FR.doFlashPrint.createCallback(self.currentSessionID))['catch'](WT.toastError);
        });
    },

    /**
     * applet打印
     */
    appletPrint: function () {
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
            }).then(FR.doAppletPrint.createCallback(self.currentSessionID))['catch'](WT.toastError);
        });
    },

    pdfPrint: function (isPopup) {
        if (this.fireEvent(FR.Events.BPDFPRINT) === false) {
            return;
        }
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
                if (FR.Browser.isIE()) {
                    FR.doPDFPrint(self.currentSessionID, isPopup);
                } else {
                    // james：pdfPrint需要用format=pdfPrint，服务器才能识别为是打印状态啊
                    window.open(FR.servletURL + "?op=fr_pdfprint&cmd=native&sessionID=" + self.currentSessionID, "_blank");
                }
            })['catch'](WT.toastError).then(function () {
                self.fireEvent(FR.Events.APDFPRINT);
            });
        });
    },

    /**
     * 零客户端打印
     */
    noClientPrint: function (popupMarginSetup, ieQuietPrint) {
        FR.doNoClientPrint(this.currentSessionID, popupMarginSetup, ieQuietPrint);
    },

    /**
     * (新)本地打印
     */
    newNativePrint: function (popupDialog, popupSheetSelectDialog, serverURL) {
        var self = this;
        FR.doNewNativePrint(this.currentSessionID, {
            //是否弹窗
            isPopUp: popupDialog,
            currentSheetIndex: self.selectedIndex + 1,  // 从 1 开始计数
            isSheetSelectPopUp: popupSheetSelectDialog,
            currentPageIndex: self.currentPageIndex,
            serverURL: serverURL
        });
    },

    /**
     * 导出pdf
     * @param extype
     */
    exportReportToPDF: function (extype) {
        if (this.fireEvent(FR.Events.BTOPDF) === false) {
            return;
        }
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
                WT.net.exportPDF(extype);
            })['catch'](WT.toastError).then(function () {
                self.fireEvent(FR.Events.ATOPDF);
            });
        });
    },

    /**
     * 导出Excel
     *
     *     @example
     *     _g().exportReportToExcel('simple');//原样导出excel
     *     _g().exportReportToExcel('page', [1]);//分页导出excel，并且只导出第二个sheet
     *
     * @param extype 导出方式
     * @param sheets 导出的sheet序号数组
     */
    exportReportToExcel: function (extype, sheets) {
        if (this.fireEvent(FR.Events.BTOEXCEL) === false) {
            return;
        }
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
                WT.net.exportExcel(extype, sheets);
            })['catch'](WT.toastError).then(function () {
                self.fireEvent(FR.Events.ATOEXCEL);
            });
        });
    },

    directExportToExcel: function (dsName, fileName, params, colNames) {
        if (this.fireEvent(FR.Events.BTOEXCEL) === false) {
            return;
        }

        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
                WT.net.directExportToExcel(dsName, fileName, params, colNames);
            })['catch'](WT.toastError).then(function () {
                self.fireEvent(FR.Events.ATOEXCEL);
            });
        });
    },

    /**
     * 导出word
     */
    exportReportToWord: function () {
        if (this.fireEvent(FR.Events.BTOWORD) === false) {
            return;
        }
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
                WT.net.exportWORD();
            })['catch'](WT.toastError).then(function () {
                self.fireEvent(FR.Events.ATOWORD);
            });
        });
    },

    /**
     * 导出图片
     *
     *     @example
     *     contentPane.exportReportToImage('png'); //导出png类型的图片
     *
     * @param extype
     */
    exportReportToImage: function (extype) {
        if (this.fireEvent(FR.Events.BTOIMAGE) === false) {
            return;
        }
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
                WT.net.exportImage(extype);
            })['catch'](WT.toastError).then(function () {
                self.fireEvent(FR.Events.ATOIMAGE);
            });
        });
    },

    /**
     * 服务器端打印
     * @param quietConfig 可选
     *
     * 调用举例：
     * window.contentPane.printReportServer({
     *     pageType: 2,  // 打印页码类型：0：所有页，1：当前页，2：指定页
     *     pageIndex: '1-3',  // 页码范围。当 pageType 为 2 时有效
     *     printerName: "" // 指定打印机
     * });
     */
    printReportServer: function (quietConfig) {
        if (this.fireEvent(FR.Events.BSEVERPRINT) === false) {
            return;
        }
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(function () {
                FR.doServerPrint(self.currentSessionID, self.currentPageIndex, quietConfig);
            }).then(function () {
                self._afterContentSave();
            })['catch'](WT.toastError).then(function () {
                self.fireEvent(FR.Events.ASERVERPRINT);
            });
        });
    },

    /**
     * 设置打印偏移
     */
    SetPrinterOffset: function () {
        FR.showIframeDialog({
            title: FR.i18nText("Fine-Engine_Print_Set_Printer_Offset"),
            width: Math.max(FR.i18nTextWidth("Fine-Engine_X_Offset", 7) * 2 + 280, 420),
            height: 300,
            url: FR.servletURL + "?op=printer_offset&cmd=pt_open&sessionID=" + this.currentSessionID
        });
    },

    /**
     * 发送邮件
     */
    emailReport: function (jo) {
        if (this.fireEvent(FR.Events.BEMAIL) === false) {
            return;
        }
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
            }).then(FR.showEmailDialog.createCallback({
                sessionID: self.currentSessionID,
                emailAttr: FR.jsonDecode(jo),
                onFinish: function () {
                    self.fireEvent(FR.Events.AEMAIL);
                }
            }))['catch'](function () {
                self.fireEvent(FR.Events.AEMAIL);
            });
        });
    },

    /**
     * 初始化excel导入按钮
     * 自己创建一个按钮并把他作为excel导入按钮:
     *
     *     @example
     *     var btn = new FR.Button({text:'excel导入'});
     *     contentPane.initExcelButton(btn);
     *
     * @param button
     */
    initExcelButton: function (button, formid, type) {
        //wei : 为了点击按钮就能弹出文件选择框，在按钮的位置放置了一个透明的文件上传控件。
        var importExcelButton = button.getRootElement();
        importExcelButton.one("mouseover", function () {
            if ($("#" + formid).length > 0) {
                var $uploadForm = $("#" + formid);
                var input = $("input", $uploadForm);
            } else {
                var $uploadForm = $('<form enctype="multipart/form-data" class="class_importexcelform" id="' + formid + '"></form>').appendTo("body");
                var input = $('<input type="file" name="file" style="width:86px; height:23px; cursor : pointer" size="2"/>').appendTo($uploadForm);
            }
            input.mouseover(
                function () {
                    importExcelButton.mouseover();
                }).mouseout(
                function () {
                    importExcelButton.mouseout();
                }).mousedown(
                function () {
                    importExcelButton.mousedown();
                }).mouseup(
                function () {
                    importExcelButton.mouseup();
                }).change(function () {
                if (!button.isEnable()) {
                    return;
                }
                if (this.value === "") {
                    return;//ie将value改成""的时候也会触发change事件
                }
                _g()._importExcelData(formid, type);
            }).click(function (e) {
                if (!button.isEnable()) {
                    e.stopEvent();
                }
                //neil:多次导入同一个excel值不变的bug 21287
                this.value = "";
            });
            input.css({
                top: importExcelButton.offset().top,
                left: importExcelButton.offset().left,
                width: importExcelButton.width(),
                display: type === 'menu' ? 'none' : '',
                position: 'absolute',
                'z-index': 7999,
                'opacity': 0
            });
        });
    },
    _importExcelData_Customize: function (formid) {
        this._importExcelData(formid, 'customize');
    },
    _importExcelData_Customize_Menu: function (formid) {
        this.importExcelType = 'customize_m';
        $('#' + formid + ' input').click();
    },
    _importExcelData_Cover_Menu: function (formid) {
        this.importExcelType = 'cover';
        $('#' + formid + ' input').click();
    },
    _importExcelData_Clean_Menu: function (formid) {
        this.importExcelType = 'clean';
        $('#' + formid + ' input').click();
    },
    _importExcelData_Append_Menu: function (formid) {
        this.importExcelType = 'append';
        $('#' + formid + ' input').click();
    },
    _importExcelData_Multi: function (formid, type) {
        this.importExcelType = type;
        this._importExcelData(formid, type);
    },
    importExcelData: function (formid) {
        this._importExcelData(formid);
    },
    /**
     * excel导入
     */
    _importExcelData: function (formid, type) {
        var self = this;
        var rtype = type === 'menu' ? this.importExcelType : type;
        var $form = $('#' + formid);
        if ($form.length === 0) {
            return;
        }
        var $file = $('input:file', $form);
        if (!$file.val()) {
            FR.Msg.toast(FR.i18nText("Fine-Engine_Function_The_Selected_File_Cannot_Be_Null") + '!');
            return;
        }

        self.fireEvent(FR.Events.BIMEXCEL);
        FR.showLoadingDialog({
            width: 310,
            height: 80
        });
        if (rtype && rtype.indexOf('customize') !== -1) {
            WT.net.customizeImportExcel($form).then(function (str) {
                var result = FR.jsonDecode(str.data);
                if (result.success) {
                    FR.$defaultImport('/com/fr/write/web/excel/ces.js', 'js');
                    FR.$defaultImport('/com/fr/write/web/css/ces.css', 'css');
                    self._stopEdit(false);
                    FR.hideLoadingDialog();
                    FR.CUSTEXCELIMPORT.initSettingDialog(result.msg, self.currentSessionID, function (ress, status) {
                        ress.data = ress.responseText;
                        var res = WT.dealWithHtmlResult(ress);
                        var resData = res.data;
                        self._importOver(resData);
                        self.fireEvent(FR.Events.AIMEXCEL);
                    });
                } else {
                    FR.Msg.toast(FR.i18nText(result.msg));
                }
            })['catch'](function (err) {
                if (err !== undefined) {
                    FR.Msg.toast(err);
                } else {
                    FR.Msg.toast(FR.i18nText("Fine-Engine_Send_Failed"));
                }
            });
        } else {
            WT.net.importExcel($form, rtype).then(function (str) {
                self._importOver(str);
            })['catch'](function () {
                FR.Msg.toast(FR.i18nText("Fine-Engine_Send_Failed"));
            }).then(function () {
                FR.hideLoadingDialog();
                self.fireEvent(FR.Events.AIMEXCEL);
            });
        }
    },
    _importOver: function (str) {
        var self = this;
        var result = FR.jsonDecode(str);
        if (result.success) {
            //导入excel，设置了自动暂存就暂存
            if (self.options.isAutoStash) {
                self.stash();
            }
            if (FR.Browser.isIE()) {
                //neil:为什么要这么蛋疼呢, 因为file这货在ie里面是readonly的, 不能随意改变值, replace掉
                var $uploadForms = $(".class_importexcelform");
                $.each($uploadForms, function (idx, item) {
                    var input = $("input", item);
                    input.replaceWith(input.clone(true));
                });
            }
            WT.net.refreshReport().then(function () {
                self.sheetContainer.clearDirtyCellsAll();
                self.sheetContainer.clearOtherSheets();
                self.reloadSheet();
            });
        } else {
            FR.Msg.toast(FR.i18nText(result.msg));
        }
    },
    /**
     * 对外接口,增量导入
     */
    importExcel_Append: function () {
        this._importExcel('append');
    },
    /**
     * 对外接口,覆盖导入
     */
    importExcel_Cover: function () {
        this._importExcel('cover');
    },
    /**
     * 对外接口,清空导入
     */
    importExcel_Clean: function () {
        this._importExcel('clean');
    },
    importExcel: function () {
        this._importExcel();
    },
    _importExcel: function (type) {
        var self = this;
        var url = FR.servletURL + "?sessionID=" + FR.SessionMgr.getSessionID() +
            "&op=fr_write&cmd=imp_w_excel_data";
        url = type ? (url + '&type=' + type) : url;
        FR.showUploadDialog({
            title: FR.i18nText("Fine-Engine_Report_Import_Excel_Import"),
            autoUpload: true,
            url: url,
            callback: function (res, status) {
                if (status === 'success') {
                    var result = FR.jsonDecode(res.responseText);
                    if (result.success) {
                        WT.net.refreshReport().then(function () {
                            self.sheetContainer.clearDirtyCellsAll();
                            self.sheetContainer.clearOtherSheets();
                            self.reloadSheet();
                        })['catch'](WT.emptyFn);

                        //导入excel，设置了自动暂存就暂存
                        if (self.options.isAutoStash) {
                            self.stash();
                        }
                        if (FR.Browser.isIE()) {
                            //neil:为什么要这么蛋疼呢, 因为file这货在ie里面是readonly的, 不能随意改变值, replace掉
                            var $uploadForms = $(".class_importexcelform");
                            $.each($uploadForms, function (idx, item) {
                                var input = $("input", item);
                                input.replaceWith(input.clone(true));
                            });
                        }
                    } else {
                        FR.Msg.toast(FR.i18nText(result.msg));
                    }
                } else {
                    FR.Msg.toast(FR.i18nText("Fine-Engine_Send_Failed"));
                }

            },
            beforeAction: function () {
                self.fireEvent(FR.Events.BIMEXCEL);
            },
            afterAction: function () {
                self.fireEvent(FR.Events.AIMEXCEL);
            }
        });
    },

    /**
     * 工具栏增加记录
     *
     *     @example
     *     contentPane.appendReportRC(1);
     *
     * @param count 增加的行数
     * @param cellLocation 增加的格子位置 一般不传这个参数用的是当前焦点所在格子
     */
    appendReportRC: function (count, cellLocation) {
        var self = this;
        var location = {
            sheetIndex: this.getSheetIndex()
        };
        if (cellLocation) {
            location.column = cellLocation.column;
            location.row = cellLocation.row;
        } else {
            var cell = this.getSelectedCell();
            if (cell) {
                var l = cell.getLocation();
                location.column = l.column;
                location.row = l.row;
            } else {
                FR.Msg.toast(FR.i18nText('Fine-Engine_Write_Enhanc_Please_Select_A_Cell'));
                return;
            }
        }
        if (this.fireEvent(FR.Events.BA) === false) {
            return;
        }
        var o = {
            overlay_immediately: true,
            text: FR.i18nText("Fine-Engine_Cal_Calculating") + "...."
        };
        FR.showLoadingDialog(o);
        this.stopEditAndWaitCal(true).then(function () {
            return WT.net.appendRow(location, count, false, true, self.getSheetElement());
        }).then(function (result) {
            WT.each(result.effectedSheet, function (sheetIdx) {
                if (sheetIdx !== self.getSheetIndex()) {
                    self.sheetContainer.clearSheet(sheetIdx);
                }
            });
            self.reloadSheetAfterAppendOrDelete(result.fec);
        })['catch'](WT.toastError).then(function () {
            FR.hideLoadingDialog();
            self.fireEvent(FR.Events.AA);
        });
    },

    /**
     * 插入行按钮触发的插入行
     * @param location 包含{'column', 'row'}
     * @param sheetIndex
     * @param count
     */
    appendReportRow: function (location, sheetIndex, count) {
        var self = this;
        if (WT.isString(location)) {
            var ll = FR.cellStr2ColumnRow(location);
            location = {
                column: ll.col,
                row: ll.row
            }
        }
        var l = $.extend({
            sheetIndex: sheetIndex
        }, location);
        if (this.fireEvent(FR.Events.BA) === false) {
            return;
        }
        var o = {
            overlay_immediately: true,
            text: FR.i18nText("Fine-Engine_Cal_Calculating") + "...."
        };
        FR.showLoadingDialog(o);
        this.stopEditAndWaitCal(true).then(function () {
            return WT.net.appendRow(l, count, false, false, self.getSheetElement());
        }).then(function (result) {
            WT.each(result.effectedSheet, function (sheetIdx) {
                if (sheetIdx !== self.getSheetIndex()) {
                    self.sheetContainer.clearSheet(sheetIdx);
                }
            });
            self.reloadSheetAfterAppendOrDelete(result.fec);
        })['catch'](WT.toastError).then(function () {
            FR.hideLoadingDialog();
            self.fireEvent(FR.Events.AA);
        });
    },

    /**
     * 工具栏的删除记录
     *
     *     @example
     *     contentPane.deleteReportRC();
     *
     * @param cellLocation 删除的位置 一般不传这个参数用的是当前焦点所在格子
     */
    deleteReportRC: function (cellLocation) {
        var self = this;
        var location = {
            sheetIndex: this.getSheetIndex()
        };
        if (cellLocation) {
            if (WT.isArray(cellLocation)) {
                location.crs = WT.map(cellLocation, function (l) {
                    if (WT.isString(l)) {
                        var ll = FR.cellStr2ColumnRow(l);
                        return {
                            column: ll.col,
                            row: ll.row
                        }
                    } else {
                        return {
                            column: l.column,
                            row: l.row
                        }
                    }
                });
            } else {
                if (WT.isString(cellLocation)) {
                    var ll = FR.cellStr2ColumnRow(cellLocation);
                    location.column = ll.col;
                    location.row = ll.row;
                } else {
                    location.column = cellLocation.column;
                    location.row = cellLocation.row;
                }
            }
        } else {
            var cell = this.getSelectedCell();
            if (cell) {
                var l = cell.getLocation();
                location.column = l.column;
                location.row = l.row;
            } else {
                FR.Msg.toast(FR.i18nText('Fine-Engine_Write_Enhanc_Please_Select_A_Cell'));
                return;
            }
        }
        if (this.fireEvent(FR.Events.BD) === false) {
            return;
        }
        var o = {
            overlay_immediately: true,
            text: FR.i18nText("Fine-Engine_Cal_Calculating") + "...."
        };
        FR.showLoadingDialog(o);
        this.stopEditAndWaitCal(true).then(function () {
            return WT.net.deleteRow(location, true, true, self.getSheetElement());
        }).then(function (result) {
            WT.each(result.effectedSheet, function (sheetIdx) {
                if (sheetIdx !== self.getSheetIndex()) {
                    self.sheetContainer.clearSheet(sheetIdx);
                }
            });
            self.reloadSheetAfterAppendOrDelete(result.fec);
        })['catch'](WT.toastError).then(function () {
            FR.hideLoadingDialog();
            self.fireEvent(FR.Events.AD);
        });
    },

    /**
     * 删除行按钮触发的删除行
     * @param location 包含{'column', 'row'}
     * @param sheetIndex
     */
    deleteReportRow: function (location, sheetIndex) {
        var self = this;
        if (WT.isString(location)) {
            var ll = FR.cellStr2ColumnRow(location);
            location = {
                column: ll.col,
                row: ll.row
            }
        }
        var l = $.extend({
            sheetIndex: sheetIndex
        }, location);
        if (this.fireEvent(FR.Events.BD) === false) {
            return;
        }
        var o = {
            overlay_immediately: true,
            text: FR.i18nText("Fine-Engine_Cal_Calculating") + "...."
        };
        FR.showLoadingDialog(o);
        this.stopEditAndWaitCal(true).then(function () {
            return WT.net.deleteRow(l, true, false, self.getSheetElement());
        }).then(function (result) {
            WT.each(result.effectedSheet, function (sheetIdx) {
                if (sheetIdx !== self.getSheetIndex()) {
                    self.sheetContainer.clearSheet(sheetIdx);
                }
            });
            self.reloadSheetAfterAppendOrDelete(result.fec);
        })['catch'](WT.toastError).then(function () {
            FR.hideLoadingDialog();
            self.fireEvent(FR.Events.AD);
        });
    },

    // 清空
    clear: function ($btn, showDialog) {
        if (showDialog === false) {
            WT.setWidgetEnable($btn, false);
            WT.net.clearStashed()['catch'](WT.emptyFn).then(function () {
                WT.setWidgetEnable($btn, true);
            });
        } else {
            var config = {
                title: FR.i18nText("Fine-Engine_Write_Clear"),
                text4OK: FR.i18nText("Fine-Engine_Confirm"),
                text4Cancel: FR.i18nText("Fine-Engine_Cancel"),
                onOK: function () {
                    WT.setWidgetEnable($btn, false);
                    WT.net.clearStashed().then(function (str) {
                        if (str === 'failed') {
                            return WT.createPromiseError(str);
                        }
                        FR.Msg.toast(FR.i18nText('Fine-Engine_Report_Successfully'));
                    })['catch'](function () {
                        FR.Msg.toast(FR.i18nText('Fine-Engine_Failed'));
                    }).then(function () {
                        WT.setWidgetEnable($btn, true);
                    });
                },
                destroyOnClose: true,
                width: 355,
                height: 150,
                contentWidget: {
                    type: 'absolute',
                    items: [
                        {
                            type: 'label',
                            value: FR.i18nText("Fine-Engine_Write_Are_Stashed_Clear"),
                            height: 21,
                            width: Math.max(FR.i18nTextWidth("Fine-Engine_Write_Are_Stashed_Clear", 7) + 10, 200),
                            textalign: 'left',
                            fontsize: 14,
                            color: 'gray',
                            x: 10,
                            y: 10
                        }
                    ]
                }
            };
            FR.showConfirmDialog(config);
        }
    },

    // 暂存
    stash: function ($btn, showMsg) {
        WT.setWidgetEnable($btn, false);
        var self = this;
        this.stopEditAndWaitCal().then(function () {
            WT.net.saveContent(self._getDirtyCellListAll()).then(function (str) {
                if (str !== 'success') {
                    return WT.createPromiseResult(str)
                }
                self._afterContentSave();
                return WT.net.stash();
            }).then(function () {
                if (showMsg === true) {
                    FR.Msg.toast(FR.i18nText('Fine-Engine_Report_Successfully'));
                }
            })['catch'](function () {
                if (showMsg === true) {
                    FR.Msg.alert(FR.i18nText('Fine-Engine_Write_Stash'), FR.i18nText('Fine-Engine_Write_Stash_Failed_Msg'));
                }
            }).then(function () {
                WT.setWidgetEnable($btn, true);
            })
        });
    },

    /**
     * 提交参数
     * @param widget 触发的控件
     */
    submitParameter: function (widget) {
        var self = this;
        if (!this.parameterPanel) {
            return;
        }
        if (!this.parameterPanel.verify()) {
            return;
        }
        WT.setWidgetEnable(widget, false);
        this.waitForCalculation().then(function () {
            WT.net.submitFormData(self.parameterPanel.getParameters()).then(function () {
                //提交参数成功
                self.loadReportData();
                self.parameterPanel.fireEvent(FR.Events.SS);
            })['catch'](function (e) {
                self.parameterPanel.fireEvent(FR.Events.SF);
                WT.toastError(e);
            }).then(function () {
                WT.setWidgetEnable(widget, true);
                self.parameterPanel.fireEvent(FR.Events.SC);
            });
        });
    },

    formSubmit: function () {
        if (!this.parameterPanel) {
            return;
        }
        this.parameterPanel.formSubmit.apply(this.parameterPanel, arguments);
    },

    parameterCommit: function () {
        this.submitParameter();
    },

    getWidgetByCell: function (locationStr) {
        var loc = FR.cellStr2ColumnRow(locationStr);
        return this.getCurrentSheet().getWidgetByCell(loc.col, loc.row);
    },

    getWidgetByName: function (name) {
        var widgets = this.getWidgetsByName(name);
        return widgets.length > 0 ? widgets[widgets.length - 1] : null;
    },

    getWidgetsByName: function (name) {
        if (!WT.isString(name)) {
            return [];
        }
        var sheet = this.getCurrentSheet();

        // 先同步加载所有该名称的控件
        try {
            var cellConfig = WT.net.getWidgetsByNameSync(name, this.getSheetIndex());
            sheet.appendNewCells(cellConfig);
        } catch (ignore) {
        }

        var cells = WT.filter(sheet.getCells(), function (cell) {
            return cell.hasWidget() && cell.getWidgetName() === name.toUpperCase();
        });
        return WT.map(cells, function (cell) {
            return cell.getWidget();
        });
    },

    getCellValue: function (o) {
        var reportIndex = this.getSheetIndex(), column = 0, row = 0;
        if (arguments.length === 1 && WT.isObject(o)) {
            if (o.reportIndex) {
                reportIndex = o.reportIndex;
            }
            if (o.columnIndex) {
                column = o.columnIndex;
            }
            if (o.rowIndex) {
                row = o.rowIndex;
            }
        } else if (arguments.length === 3
            && WT.isNumber(arguments[0])
            && WT.isNumber(arguments[1])
            && WT.isNumber(arguments[2])) {
            reportIndex = arguments[0];
            column = arguments[1];
            row = arguments[2];
        } else if (arguments.length === 2
            && WT.isNumber(arguments[0])
            && WT.isNumber(arguments[1])) {
            column = arguments[0];
            row = arguments[1];
        } else if (WT.isString(o)) {
            var l = FR.cellStr2ColumnRow(o);
            column = l.col;
            row = l.row;
        } else {
            throw new Error('Arguments dismatch');
        }
        var sheet = this.getSheet(reportIndex);
        var cell = sheet.getCell(column, row);
        if (!cell) {
            try {
                var cellConfig = WT.net.getWidgetsByNameSync({
                    column: column,
                    row: row,
                    sheetIndex: reportIndex
                });
                sheet.appendNewCells(cellConfig);
                cell = sheet.getCell(column, row);
            } catch (ignore) {
            }
        }
        if (cell) {
            var dirtyCellMap = this.sheetContainer.getDirtyCellMap(reportIndex);
            return dirtyCellMap[cell.getID()] ? dirtyCellMap[cell.getID()].value : cell.getValue();
        } else {
            return null;
        }
    },

    reCalCurLGPPane: function () {
        this.reloadSheet();
    },

    refreshAllSheets: function () {
        var self = this;
        WT.net.submitFormData(self.parameterPanel ? self.parameterPanel.getParameters() : '').then(function () {
            self.sheetContainer.reloadSheetAll();
        })['catch'](function (e) {
            WT.toastError(e);
        });
    },

    refreshWidgets: function () {
        // ignore
    },

    reloadCurLGPPane: function (fec) {
        if (fec) {
            this.reloadSheetAfterAppendOrDelete({
                column: fec.col,
                row: fec.row
            });
        } else {
            this.reloadSheet();
        }
    },

    setCellValue: function (o) {
        var reportIndex = this.getSheetIndex(), column = 0, row = 0, value;
        if (arguments.length === 1 && WT.isObject(o)) {
            if (o.reportIndex) {
                reportIndex = o.reportIndex;
            }
            if (o.columnIndex) {
                column = o.columnIndex;
            }
            if (o.rowIndex) {
                row = o.rowIndex;
            }
            value = o.value;
        } else if (arguments.length === 3) {
            if (WT.isNumber(arguments[0]) && WT.isNumber(arguments[1])) {
                column = arguments[0];
                row = arguments[1];
                value = arguments[2];
            } else if (WT.isString(arguments[0])) {
                var l = FR.cellStr2ColumnRow(arguments[0]);
                column = l.col;
                row = l.row;
                value = arguments[2];
            }
        } else {
            throw new Error('Arguments dismatch');
        }
        this.sheetContainer.setCellValue(value, column, row, reportIndex);
    },

    setEditOnClick: function (editOnClick) {
        this.globalConfig.editOnClick = editOnClick;
    },

    setEditOnMove: function (editOnMove) {
        this.globalConfig.editOnMove = editOnMove;
    },

    setFireStopEditOnChange: function (fireOnChange) {
        this.globalConfig.fireOnChange = fireOnChange;
    },

    stopEditing: function () {
        return this._stopEdit(true);
    },

    writeCellValue: function (tdCell) {
        // ignore
        /*
         throw new Error(FR.i18nText('FR-Designer_Write_Enhance_Preview')
         + FR.i18nText('Fine-Engine_Write_Enhanc_Not_Support_Function')
         + '"writeCellValue"')
         */
    },

    makeHighlight: function () {
        // ignore
    },

    /**
     *  删除记录
     */
    deleteRows: function () {
        this.deleteReportRC.apply(this, arguments);
    },

    /**
     * 增加记录
     */
    appendRows: function () {
        this.appendReportRC.apply(this, arguments);
    },
    /**
     * 加一个空的savereport,按钮的填报事件是要先做savereport的,新填报是自动实时save不需要业务逻辑,这个函数只是为了按钮那边的提交业务正常执行
     * @param fn
     */
    saveReport: function (fn) {
        fn = fn || FR.emptyFn;
        fn();
    }

});
WT.ReportPage.EVENT_CALCULATING_FINISH = 'EVENT_CALCULATING_FINISH';

/**
 * 用来模拟原填报下的currLGP对象，实现原有功能
 * 注：该对象下所有涉及到jQuery对象（比如td）的，都为模拟属性，直接操作该jQuery对象对单元格无效，应该仅作为调用其他函数的参数使用
 */
WT.WLGP = FR.extend(FR.OB, {

    _init: function () {
        WT.WLGP.superclass._init.apply(this, arguments);
        var self = this;
        this.report = this.options.report;
        this.write = this.report;
        this.writePane = this.report;
        this.report.sheetContainer.on(WT.SheetContainer.EVENT_SHEET_CHANGE, function () {
            self.$container = self.report.getSheetElement();
            self.sheet = self.report.getCurrentSheet();
        });
        this._bindEvents();
    },

    _bindEvents: function () {
        var self = this;
        this._relayEvents([
            FR.Events.RESIZE,
            FR.Events.STARTLOAD,
            FR.Events.AFTERLOAD,
            FR.Events.BVW,
            FR.Events.AV,
            FR.Events.BW,
            FR.Events.AVW,
            FR.Events.AW,
            FR.Events.WS,
            FR.Events.WF,
            FR.Events.BPDFPRINT,
            FR.Events.APDFPRINT,
            FR.Events.BTOPDF,
            FR.Events.ATOPDF,
            FR.Events.BTOEXCEL,
            FR.Events.ATOEXCEL,
            FR.Events.BTOWORD,
            FR.Events.ATOWORD,
            FR.Events.BTOIMAGE,
            FR.Events.ATOIMAGE,
            FR.Events.BSEVERPRINT,
            FR.Events.ASERVERPRINT,
            FR.Events.BPRINT,
            FR.Events.APRINT,
            FR.Events.BEMAIL,
            FR.Events.AEMAIL,
            FR.Events.BIMEXCEL,
            FR.Events.AIMEXCEL,
            FR.Events.BA,
            FR.Events.AA,
            FR.Events.BD,
            FR.Events.AD
        ]);
        this.report.sheetContainer.on(WT.Cell.EVENT_CELL_SELECT, function (cell) {
            self.fireEvent(WT.Cell.EVENT_CELL_SELECT, self._getTDCellByCell(cell));
        });
    },

    _relayEvents: function (events) {
        var self = this;
        WT.each(events, function (event) {
            self.report.on(event, function () {
                self.fireEvent.apply(self, Array.prototype.concat.apply([event], arguments))
            });
        });
    },

    _getCellByTDCell: function (tdCell) {
        var col = this.getTDCol(tdCell) - 1;
        var row = this.getTDRow(tdCell) - 1;
        if (col >= 0 && row >= 0) {
            return this.sheet.getCell(col, row);
        }
        return null;
    },

    _getTDCellByCell: function (cell) {
        if (!cell) {
            return null;
        }
        var loc = cell.getLocation();
        var sheetIndex = this.report.getSheetIndex();
        var htmlStr = cell.getRawHtmlStrValue();
        return $('<td id="'
            + cell.getID()
            + '" col="'
            + (loc.column + 1)
            + '" row="'
            + (loc.row + 1)
            + '" sheet="'
            + sheetIndex
            + '">'
            + htmlStr
            + '</td>');
    },

    calCurrentPage: function () {
        this.report.reloadSheet();
    },

    displayTDCell: function (tdCell, cv, presentValue) {
        var cell = this._getCellByTDCell(tdCell);
        if (!cell) {
            return;
        }
        cell.setValue(presentValue, true, cv);
    },

    formatContent: function (dp, $tdCell) {
        var cell = this._getCellByTDCell($tdCell);
        if (!cell) {
            return;
        }
        cell.setValue(dp, true);
        return cell.getRawHtmlStrValue();
    },

    getTDCell: function (col, row) {
        var sheetIndex = this.report.getSheetIndex();
        var cell = this.sheet.getWidgetByCell(col, row);
        this._getTDCellByCell(cell);
    },

    getTDCol: function (td) {
        var col = parseInt(td.attr('col'));
        return isNaN(col) ? -1 : col;
    },

    getTDRow: function (td) {
        var row = parseInt(td.attr('row'));
        return isNaN(row) ? -1 : row;
    },

    hideSelectFrame: function () {
        var sheet = this.report.getCurrentSheet();
        if (sheet) {
            sheet.disableBorder();
        }
    },

    isEditableWidget: function (tdCell) {
        var cell = this._getCellByTDCell(tdCell);
        if (cell) {
            return cell.hasEditorWidget();
        }
        return false;
    },

    isFrozen: function () {
        var sheet = this.report.getCurrentSheet();
        if (sheet) {
            return sheet.isFrozen();
        }
        return false;
    },

    isWeakSelectable: function (tdCell) {
        return this.isEditableWidget(tdCell);
    },

    modifyRowHeightAfterContentChange: function ($tr, oriHeight) {
        //
    },

    moveTDCell: function (direction) {
        var sheet = this.report.getCurrentSheet();
        if (!sheet) {
            return;
        }
        switch (direction) {
            case 1:
                sheet.selectUp();
                break;
            case 2:
                sheet.selectDown();
                break;
            case 3:
                sheet.selectPrev();
                break;
            case 4:
                sheet.selectNext();
                break;
        }
    },

    present: function ($tdCell, cv) {
        var cell = this._getCellByTDCell($tdCell);
        if (!cell) {
            return;
        }
        cell.setValue(cv, true);
    },

    saveCurrentPage: function () {
        //
    },

    setCellValue: function () {
        var reportIndex = this.report.getSheetIndex();
        var column = 0, row = 0;
        var value;
        if (arguments.length >= 3) {
            value = arguments[2];
            if (WT.isNumber(arguments[0]) && WT.isNumber(arguments[1])) {
                column = arguments[0];
                row = arguments[1];
            } else if (WT.isString(arguments[0])) {
                var l = FR.cellStr2ColumnRow(arguments[0]);
                column = l.col;
                row = l.row;
            }
        } else {
            return;
        }
        this.report.sheetContainer.setCellValue(value, column, row, reportIndex);
    },

    getCellValue: function () {
        return this.report.getCellValue.apply(this.report, arguments);
    }

});