/**
 * 报表容器
 * 组织多个sheet的显示切换，负责tab的控制
 * @extends FR.OB
 *
 * @cfg {JSON} options 配置属性
 * @cfg {String} options.tabPosition sheet标签页显示位置，值为'top'或'bottom'
 * @cfg {Boolean} options.isAlignLeft 报表显示位置是否靠左
 * @cfg {Boolean} options.isShowWidgets 直接显示控件
 */
WT.SheetContainer = FR.extend(FR.OB, {

    _defaultConfig: function () {
        return $.extend(WT.SheetContainer.superclass._defaultConfig.apply(), {
            isAlignLeft: true,
            isShowWidgets: false,
            tabPosition: 'bottom',
            noControl: false,
            defaultActiveIndex: 0,
            sheets: [],
            report: null,
            defaultShow: false
        });
    },

    _init: function () {
        WT.SheetContainer.superclass._init.apply(this, arguments);
        this.needToLoadReport = false;
        this._initView();
    },

    /**
     * 初始化dom以及子控件dom
     */
    _initView: function () {
        this.element = $('<div class="wt-absolute-layout"></div>');
        this._createSheetView(this.options.sheets);
        if (this.options.defaultShow === false) {
            WT.nextTick(function () {
                //这边为啥延迟呢?因为new WT.TabBar的时候会延迟1ms初始化(工具栏那边的显示单元格值的控件需要等单元格初始化出来之后才能赋值),
                // 初始化里面取需要获取tab尺寸,必须在初始化之后隐藏,否则获取不到尺寸
                this.getRootElement().hide();
            }, this);
        }
    },

    /**
     * 创建sheet的内容
     * @private
     */
    _createSheetView: function (sheets) {
        var self = this, o = this.options;
        if (FR.isEmpty(this.activeIndex)) {
            this.activeIndex = o.defaultActiveIndex;
        }
        if (this.activeIndex > sheets.length - 1) {
            this.activeIndex = 0;
        }
        this.sheets = WT.map(sheets, function (sheet, idx) {
            var s = new WT.Sheet({
                isAlignLeft: o.isAlignLeft,
                isShowWidgets: o.isShowWidgets,
                name: sheet.title,
                sheetIndex: idx,
                store: o.store
            });
            s.on(FR.Events.STARTLOAD, function () {
                self._handleLoadStart(idx);
                self.fireEvent(FR.Events.STARTLOAD, idx);
            });
            s.on(FR.Events.AFTERLOAD, function () {
                self._handleLoadEnd(idx);
                self.fireEvent(FR.Events.AFTERLOAD, idx);
            });
            s.on(WT.Cell.EVENT_CELL_SELECT, function (cell) {
                self._handleCellSelect(cell);
            });
            s.on(WT.Cell.EVENT_SYNC_VALUE, function (cell) {
                self._handleCalculateCellValue(cell);
            });
            s.on(WT.ReportBlock.EVENT_CELLS_RELOAD, function (cellInfoList) {
                self.setCellsValue(cellInfoList, false);
            });
            s.on(WT.ReportBlock.EVENT_LOADING_COVER, function (show) {
                if (show) {
                    self._handleLoadStart();
                } else {
                    self._handleLoadEnd();
                }
            });
            // 如果滚动条位置已经存在就保留
            if (!FR.isEmpty(sheet.centerScrollPosition)){
                s.centerScrollPosition = sheet.centerScrollPosition;
            }
            self.element.append(s.element);
            return s;
        });
        if (this.sheets.length > 1) {
            this.tabBar = new WT.TabBar({
                items: WT.map(this.sheets, function (sheet) {
                    return sheet.getName();
                }),
                tabPosition: o.tabPosition,
                noControl: o.noControl,
                defaultActiveIndex: this.activeIndex,
                store: o.store
            });
            this.element.append(this.tabBar.element);
            this.tabBar.on(FR.Events.TABCHANGE, function (index) {
                self._handleSheetSelect(index);
            });
        } else {
            //不显示tabbar时也要选中sheet
            WT.nextTick(function () {
                this._handleSheetSelect(0);
            }, this);
        }
        if (this.size) {
            this._resizeSheetAndTabBar();
        }
    },

    _resizeSheetAndTabBar: function () {
        var sheetSize = {
            x: 0,
            y: 0,
            width: this.size.width,
            height: this.size.height
        };
        if (this.tabBar) {
            var tabBarSize = {
                x: 0,
                y: 0,
                width: this.size.width,
                height: this.size.height
            };
            var tabBarHeight = this.tabBar.getHeight();
            sheetSize.height -= tabBarHeight;
            tabBarSize.height = tabBarHeight;
            if (this.options.tabPosition === 'bottom') {
                tabBarSize.y = this.size.height - tabBarHeight;
            } else {
                sheetSize.y = tabBarHeight;
            }
            this.tabBar.resize(tabBarSize.x, tabBarSize.y, tabBarSize.width, tabBarSize.height)
        }
        if (this.sheets) {
            WT.each(this.sheets, function (sheet) {
                sheet.resize(sheetSize.x, sheetSize.y, sheetSize.width, sheetSize.height);
            });
        }
    },

    /**
     * 处理显示sheet，包括首次加载默认sheet和tab上选中该sheet
     * @private
     */
    _handleSheetSelect: function (index) {
        if (!this.sheets || !this.sheets[index]) {
            return;
        }
        this.activeIndex = index;
        WT.each(this.sheets, function (sheet, idx) {
            if (index === idx) {
                if (sheet.isVisible()) {
                    //已经是选中状态了
                    return;
                }
                sheet.setVisible(true);
                //已经提交了参数并且sheet还没有加载
                if (this.needToLoadReport && !sheet.isLoaded()) {
                    sheet.loadSheetData();
                }
            } else {
                if (!sheet.isVisible()) {
                    //已经是非选中状态了
                    return;
                }
                sheet.setVisible(false);
            }
        }, this);
        this.fireEvent(WT.SheetContainer.EVENT_SHEET_CHANGE, this.activeIndex);
    },

    _handleLoadStart: function () {
        if (this.loadingCover) {
            return;
        }
        this.loadingCover = new WT.LoadingCover();
        this.loadingCover.resize(this.size.width, this.size.height);
        this.loadingCover.mount(this.element);

        // var sheet = this.getActiveSheet();
        // sheet && sheet.lockScroll(true);
    },

    _handleLoadEnd: function () {
        if (this.loadingCover) {
            this.loadingCover.destroy();
            this.loadingCover = null;

            // var sheet = this.getActiveSheet();
            // sheet && sheet.lockScroll(false);
        }
    },

    _handleCellSelect: function (cell) {
        this.fireEvent(WT.Cell.EVENT_CELL_SELECT, cell);
    },

    _handleCalculateCellValue: function (cell) {
        var self = this, o = this.options;
        this.fireEvent(WT.SheetContainer.EVENT_CALCULATING_CELL, true);
        var loc = cell.getLocation();
        var v = cell.getValue();
        if ((!cell.widget || !WT.isTreeWidget(cell.widget)) && cell.originValue) {
            v = cell.originValue;
        }
        var dirtyCellList = [{
            sheetIndex: this.activeIndex,
            cells: [{
                column: loc.column,
                row: loc.row,
                value: v
            }]
        }];
        WT.net.calculateCellValue(loc, dirtyCellList).then(function (relatedCells) {
            var cm = {};
            WT.each(relatedCells, function (c) {
                var loc = c.location;
                delete c.location;
                var listOfSheet = cm[loc.sheetIndex];
                if (!listOfSheet) {
                    cm[loc.sheetIndex] = [];
                    listOfSheet = cm[loc.sheetIndex]
                }
                c.column = loc.column;
                c.row = loc.row;
                c.depended = true;
                listOfSheet.push(c);
            });
            WT.each(cm, function (listOfSheet, key) {
                var idx = parseInt(key);
                var sheet = self.sheets[idx];
                if (sheet) {
                    sheet.applyNewValue(listOfSheet);
                }
            })
        })['catch'](WT.toastError).then(function () {
            self.fireEvent(WT.SheetContainer.EVENT_CALCULATING_CELL, false);
        }).then(function () {
            o.report.trigAutoStash();
        });
    },

    getSheetsInfo: function () {
        return WT.map(this.sheets, function (sheet, idx) {
            return {
                name: sheet.getName(),
                loaded: sheet.isLoaded(),
                active: idx === this.activeIndex
            }
        }, this)
    },
    /**
     * 定位到指定位置并聚焦
     * @param sheetIndex
     * @param cellInfo {{id, rect, blockType, name}}
     * @param needSelect
     */
    locateAndSelectCell: function (sheetIndex, cellInfo) {
        this.locateToCell(sheetIndex, cellInfo, true);
    },

    /**
     * 定位到指定位置
     * @param sheetIndex
     * @param cellInfo {{id, rect, blockType, name}}
     * @param needSelect
     */
    locateToCell: function (sheetIndex, cellInfo, needSelect) {
        var rect = cellInfo.rect;
        var blockType = cellInfo.blockType;
        if (sheetIndex !== this.activeIndex) {
            this.selectSheet(sheetIndex);
        }
        var sheet = this.sheets[sheetIndex];
        if (sheet) {
            if (needSelect) {
                var loc = FR.cellStr2ColumnRow(cellInfo.name);
                sheet.selectCell(loc.col, loc.row);
            }
            sheet.scrollIntoView(rect, blockType);
        }
    },

    getActiveSheet: function () {
        return this.sheets[this.activeIndex];
    },

    getDirtyCells: function (sheetIndex) {
        var sheet = this.sheets[sheetIndex];
        if (sheet) {
            return sheet.getDirtyCells();
        }
        return null;
    },

    getDirtyCellMap: function (sheetIndex) {
        var sheet = this.sheets[sheetIndex];
        if (sheet) {
            return sheet.getDirtyCellMap();
        }
        return null;
    },

    getDirtyCellsAll: function () {
        return WT.map(this.sheets, function (sheet) {
            return sheet.getDirtyCells();
        }, this);
    },

    clearDirtyCells: function (sheetIndex) {
        var sheet = this.sheets[sheetIndex];
        sheet && sheet.clearDirtyCells();
    },

    clearDirtyCellsAll: function () {
        WT.each(this.sheets, function (sheet) {
            sheet.clearDirtyCells();
        });
    },

    selectSheet: function (sheetIndex) {
        if (sheetIndex < 0 || sheetIndex > this.sheets.length - 1) {
            return;
        }
        this.tabBar && this.tabBar.selectTab(sheetIndex, true);
    },

    /**
     * 设置校验失败信息
     * @param infoList [{sheetIndex, cells: [{id, msg}]}]
     */
    setInvalidInfo: function (infoList) {
        WT.each(infoList, function (info) {
            this.tabBar && this.tabBar.setInvalidTab(info.sheetIndex);
            var sheet = this.sheets[info.sheetIndex];
            sheet.setInvalidCellInfo(info.cells);
        }, this);
    },

    clearInvalidInfo: function () {
        this.tabBar && this.tabBar.clearInvalidTab();
        WT.map(this.sheets, function (sheet) {
            sheet.clearInvalidCellInfo();
        }, this);
    },

    /**
     * 根据位置获取单元格
     * @param location 包含{sheetIndex, column, row}
     */
    getCell: function (location) {
        var sheet = this.sheets[location.sheetIndex];
        if (sheet) {
            return sheet.getCell(location.column, location.row);
        } else {
            return null;
        }
    },

    /**
     * 获取选中的单元格
     * @param sheetIndex
     */
    getSelectedCell: function (sheetIndex) {
        var sheet = this.sheets[sheetIndex];
        if (sheet) {
            return sheet.getSelectedCell();
        } else {
            return null;
        }
    },

    /**
     * 获取sheet的根元素
     * @param sheetIndex
     * @return {null}
     */
    getSheetElement: function (sheetIndex) {
        var sheet = this.sheets[sheetIndex];
        if (sheet) {
            return sheet.element;
        } else {
            return null;
        }
    },
    setCellsValue: function (cellInfoList, isTrig) {
        if (cellInfoList) {
            var self = this;
            WT.each(cellInfoList, function (cellInfo, idx) {
                self.setCellValue(cellInfo.value, cellInfo.location.column, cellInfo.location.row, cellInfo.location.sheetIndex, isTrig);
            });
        }
    },

    setCellValue: function (value, column, row, sheetIndex, isTrig) {
        var cell = this.getCell({
            sheetIndex: sheetIndex,
            column: column,
            row: row
        });
        if (cell) {
            if (isTrig == undefined) {//null or undefined  默认触发模板重新计算
                isTrig = true;
            }
            //单元格在屏幕范围内
            cell.setValue(value, isTrig);
        }
        // 这里如果是回填内容,就不应该再触发模版重新计算.再计算一遍导致公式失效.
        else if (isTrig !== false) {
            var self = this;
            var dirtyCellList = [{
                sheetIndex: sheetIndex,
                cells: [{
                    column: column,
                    row: row,
                    value: value
                }]
            }];
            WT.net.calculateCellValue({
                sheetIndex: sheetIndex,
                column: column,
                row: row
            }, dirtyCellList, value)['catch'](WT.alertError).then(function (relatedCells) {
                var cm = {};
                WT.each(relatedCells, function (c) {
                    var loc = c.location;
                    delete c.location;
                    var listOfSheet = cm[loc.sheetIndex];
                    if (!listOfSheet) {
                        cm[loc.sheetIndex] = [];
                        listOfSheet = cm[loc.sheetIndex]
                    }
                    c.column = loc.column;
                    c.row = loc.row;
                    c.depended = true;
                    listOfSheet.push(c);
                });
                WT.each(cm, function (listOfSheet, key) {
                    var idx = parseInt(key);
                    var sheet = self.sheets[idx];
                    if (sheet) {
                        sheet.applyNewValue(listOfSheet);
                    }
                })
            })['catch'](WT.toastError);
        }
    },

    /**
     * 清空sheet，标记sheet需要重新计算
     * 不立即计算而是等到下次切换到该sheet时再加载
     * @param sheetIndex
     */
    clearSheet: function (sheetIndex) {
        var sheet = this.sheets[sheetIndex];
        if (sheet && sheet.isLoaded()) {
            sheet.clearSheetData();
        }
    },

    clearOtherSheets: function () {
        var self = this;
        WT.each(this.sheets, function (sheet, idx) {
            if (idx !== self.activeIndex) {
                sheet.clearSheetData();
            }
        });
    },

    reloadSheet: function (sheetIndex) {
        var sheet = this.sheets[sheetIndex];
        if (sheet && sheet.isLoaded()) {
            sheet.clearSheetData();
            sheet.loadSheetData();
        }
    },

    reloadSheetAfterAppendOrDelete: function (sheetIndex, appendLocation) {
        var sheet = this.sheets[sheetIndex];
        if (sheet && sheet.isLoaded()) {
            sheet.clearSheetData();
            sheet.loadSheetDataAfterAppendOrDelete(appendLocation);
        }
    },

    reloadSheetAll: function () {
        var currentIndex = this.activeIndex;
        WT.each(this.sheets, function (sheet) {
            if (sheet.isLoaded()) {
                sheet.clearSheetData();
                if (sheet.options.sheetIndex === currentIndex) {
                    sheet.loadSheetData();
                }
            }
        });
    },

    /**
     * 设置大小
     * @param x {Number} 左边距
     * @param y {Number} 上边距
     * @param width {Number} 宽度
     * @param height {Number} 高度
     */
    resize: function (x, y, width, height) {
        this.size = {
            left: x,
            top: y,
            width: width,
            height: height
        };
        this.element.css({
            left: x,
            top: y,
            width: width,
            height: height
        });
        this.loadingCover && this.loadingCover.resize(this.size.width, this.size.height);
        this._resizeSheetAndTabBar();
    },

    /**
     * 通知报表开始加载数据
     */
    loadSheetData: function () {
        this.getRootElement().show();
        if (this.needToLoadReport) {
            this.clearSheetData();
        }
        this.needToLoadReport = true;
        if (!this.sheets) {
            return;
        }
        var currentSheet = this.sheets[this.activeIndex];
        if (!currentSheet) {
            return;
        }
        currentSheet.loadSheetData();
    },

    clearSheetData: function () {
        if (!this.sheets) {
            return;
        }
        this.element.empty();
        //重新初始化sheet组件
        this._createSheetView(this.sheets);
    },

    getRootElement: function () {
        return this.element;
    }

});
WT.SheetContainer.EVENT_SHEET_CHANGE = 'EVENT_SHEET_CHANGE';
WT.SheetContainer.EVENT_CALCULATING_CELL = 'EVENT_CALCULATING_CELL';