/**
 * 新填报下所有编辑控件的基本类
 * @class WT.BaseEditor
 * @extends WT.Widget
 *
 * @cfg {JSON} options 配置属性
 * @cfg {String} options.watermark 控件水印
 * @cfg {Boolean} options.allowBlank 允许空值
 * @cfg {String} options.errorMsg4Blank 空值时提示信息
 * @cfg {String} options.regex 填写规则
 * @cfg {String} options.errorMsg4Regex 不符合填写规则时提示信息
 */
WT.BaseEditor = FR.extend(WT.Widget, {

    TYPE: 'WT.BaseEditor',

    _defaultConfig: function () {
        return $.extend(WT.BaseEditor.superclass._defaultConfig.apply(), {
            watermark: null,
            allowBlank: true,
            regex: null
        });
    },

    /**
     * @extends WT.Widget._initVariable()
     * @protected
     */
    _initVariable: function () {
        WT.BaseEditor.superclass._initVariable.apply(this);
        /**
         * 错误信息
         * @type {string}
         */
        this.errMsg = null;

        this.isEditing = false;

        this.editedManually = false;
        this.lastEditedValue = null;
    },

    /**
     * @extends WT.Widget._initView()
     * @protected
     */
    _initView: function () {
        var o = this.options;
        this.element = this._createRootEl();
        this.element.css({
            left: o.left,
            top: o.top,
            height: o.height,
            width: o.width
        });

        this.editor = this._createEditor();
        WT.dom.appendChild(this.element, this.editor.element);
        var self = this;
        this.editor.on(BI.Input.EVENT_FOCUS, function () {
            self.editor.innerEditerBlur = false;//REPORT-12282
            self._lastValue = o.value;
            if (self._shouldRecoverValueOnFocus()) {
                self.editor.setValue(o.value);
            }
            self.isEditing = true;
            self._dealEditorStyleOnFocus();
            self.fireEvent(FR.Events.BEFOREEDIT);
            self._hideWaterMark();
            self.verify4Style(true);
            //输入时屏蔽全局快捷键监听
            if (WT.Keys.enable()) {
                WT.Keys.enable(false);
            }
        });
        this.editor.on(BI.Input.EVENT_QUICK_DOWN, function (e) {
            self._onKeyDown(e);
        });
        this.editor.on(BI.Input.EVENT_KEY_DOWN, function () {
            if (o.usedAsCell) {
                self.verify4Style(true);
            }
            self.fireEvent(FR.Events.AFTEREDIT);
        });
        this.editor.on(BI.Input.EVENT_BLUR, function (e) {
            //下拉控件点击下拉选项误触发编辑结束问题
            //REPORT-38938 REPORT-38786 
            if (!self._blurCheck()) {
                return;
            }
            self.editor.innerEditerBlur = true;//REPORT-43308 防止重复弹出
            // 切换浏览器标签页时，EVENT_BLUR会触发，但是实际上控件并没有失去焦点，手动调用一次blur
            self.editor.element.blur();//REPORT-12282
            self._dealEditorStyleOnBlur();
            self.stopEdit();
            if (o.globalConfigProvider && o.globalConfigProvider().fireOnChange) {
                if (self._lastValue !== o.value) {
                    self.fireEvent(FR.Events.STOPEDIT);
                }
            } else {
                self.fireEvent(FR.Events.STOPEDIT);
            }
            self.isEditing = false;
            if (o.format && o.format !== 'T') {
                self.editor.setValue(FR.contentFormat(o.value, o.format));
            }
            self._showWaterMark();
            //恢复全局快捷键监听
            if (!WT.Keys.enable()) {
                WT.Keys.enable(true);
            }
            self.shouldBlur = false;
        });
        this.editor.on(WT.Input.EVENT_ENTER_PRESSED, function () {
            self.fireEvent(WT.BaseEditor.EVENT_ENTER_PRESSED);
        });
        this.editor.on(WT.Input.EVENT_TAB_PRESSED, function () {
            self.fireEvent(WT.BaseEditor.EVENT_TAB_PRESSED);
        });

        this._showWaterMark();

        var v = o.value;
        if (v === null || v === undefined) {
            v = '';
        }
        this.setValue(v);

        this.setEnable(!o.disabled);
        this.setVisible(!o.invisible);
    },

    _createRootEl: function () {
        return WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-text-editor-border wt-border-box');
    },

    _createEditor: function () {
        var o = this.options;
        var editor = new WT.Input({
            element: "<input type='text' class='wt-text-editor wt-border-box'/>",
            allowBlank: true
        });
        editor.element.css({
            'font-size': o.fontSize,
            overflow: 'hidden',
            height: '100%'
        });
        return editor;
    },

    _showWaterMark: function () {
        if (this.isEditing) {
            return;
        }
        if (WT.isKey(this.options.watermark)) {
            var ieVersion = FR.Browser.getIEVersion();
            if (ieVersion !== undefined && ieVersion < 10 && this.editor.getValue() === '') {
                var self = this;
                this.placeholderLabel = $('<label class="wt-text-editor wt-border-box"></label>').css({
                    'font-size': this.options.fontSize,
                    overflow: 'hidden',
                    height: this.options.height,
                    'line-height': this.options.height + 'px',
                    width: '100%',
                    color: 'rgb(204,204,204)',
                    position: 'relative',
                    display: 'inline-block',
                    top: this.options.height * -1
                }).text(this.options.watermark).click(function (e) {
                    e.stopPropagation();
                    self.editor.focus();
                });
                WT.dom.appendChild(this.element, this.placeholderLabel);
            } else {
                this.editor.element.attr('placeholder', this.options.watermark);
            }
        }
    },

    _hideWaterMark: function () {
        this.editor.element.removeAttr('placeholder');
        if (this.placeholderLabel) {
            this.placeholderLabel.remove();
            this.placeholderLabel = undefined;
        }
    },

    /**
     * 在编辑框获取焦点时处理样式
     * @protected
     */
    _dealEditorStyleOnFocus: function () {
        this.element.addClass('wt-text-editor-border-focus');
    },

    /**
     * 在编辑框失去焦点时处理样式
     * @protected
     */
    _dealEditorStyleOnBlur: function () {
        this.element.removeClass('wt-text-editor-border-focus');
        if (this.options.usedAsCell) {
            // 仅针对单元格中的控件
            // 停止编辑时如果校验失败也不显示红色波浪线
            this._dealEditorStyleOnValid();
        } else {
            this.verify4Style(true);
        }
    },

    /**
     * 值有效时处理样式，去掉波浪线
     * @protected
     */
    _dealEditorStyleOnValid: function () {
        this.element.removeClass('wt-text-editor-border-invalid');
        this.element.removeAttr('title');
    },

    /**
     * 值无效时处理样式，设置波浪线和title值
     * @param title {String} 提示错误值
     * @protected
     */
    _dealEditorStyleOnInvalid: function () {
        this.element.addClass('wt-text-editor-border-invalid');
        this.element.attr('title', this.errMsg);
    },

    /**
     * 检验输入数据，失败会设置波浪线样式
     * @param showWarn? {Boolean} 失败时是否显示警告
     */
    verify4Style: function (showWarn) {
        //先判断是否允许空值
        var val = this.editor.getValue();
        if (!this.options.allowBlank && FR.isEmpty(val)) {
            this.errMsg = this.options.errorMsg4Blank || FR.i18nText("Fine-Engine_Base_NOT_NULL_Des");
            showWarn && this._dealEditorStyleOnInvalid();
        } else {
            if (this._verify(val)) {
                this._dealEditorStyleOnValid();
            } else {
                showWarn && this._dealEditorStyleOnInvalid();
            }
        }
    },
    /**
     * 控件校验(只校验不弹出报错)
     * @param cValue
     * @returns {*|Boolean}
     */
    verify: function (cValue) {
        return this._verify(cValue);
    },

    /**
     * 检验值，内置校验下在编辑时会调用
     * @return {Boolean} 校验成功与否
     */
    _verify: function (cValue) {
        var value = cValue !== null && cValue !== undefined
            ? cValue
            : this.options.value;
        var regex = this.options.regex;
        if (!value || value === "" || FR.isEmptyArray(value)) {
            this.errMsg = this.options.errorMsg4Blank || FR.i18nText("Fine-Engine_Base_NOT_NULL_Des");
            return this.options.allowBlank;
        }
        if (regex) {
            for (var i = 0, len = WT.BaseEditor.PluginRegex.length; i < len; i++) {
                var obj = WT.BaseEditor.PluginRegex[i];
                if (obj.text === regex) {
                    this.errMsg = this.options.errorMsg4Regex || FR.i18nText("Fine-Engine_Base_Error_Input_Value");
                    return obj.regex(value);
                }
            }

            if (typeof regex == 'string') {/* String的时候，构造一个RegExp出来*/
                try {
                    regex = new RegExp(regex);
                } catch (err) {
                    this._toastMsg("invalid Regular expressions");
                    this.errMsg = "invalid Regular expressions";
                    return false;
                }
            }
            if (!regex.test(value)) {
                /* 正则表达式是判断长度的 换行符算两个字符*/
                if (!(regex.source.indexOf(".{") > -1 && regex.test((value + '').replace(/\n/g, "aa")))) {
                    this.errMsg = this.options.errorMsg4Regex || FR.i18nText("Fine-Engine_Base_Error_Input_Value");
                    return false;
                }
            }
        }
        return true;
    },

    /**
     * 弹出提示
     * @param msg {String} 提示信息
     */
    _toastMsg: function (msg) {
        FR.Msg.toast(msg);
    },

    _shouldRecoverValueOnFocus: function () {
        return true;
    },

    _onKeyDown: function (e) {
        this.editedManually = true;
    },
    _blurCheck: function () {
        if (this.editor && this.editor.innerEditerBlur === true) {
            this.editor.innerEditerBlur = false;
            return false;
        }
        return true;
    },

    /**
     * 编辑结束，触发校验
     * 校验失败会提示错误以及恢复原值
     * @param isToastError 校验失败是否弹出错误提示
     * @return {Boolean} 校验成功与否
     */
    stopEdit: function (isToastError) {
        if (!this.editor) {
            return true;
        }
        var val = this.isEditing ? this.editor.getValue() : this.options.value;
        var result = true;
        //先判断是否允许空值
        if (!this.options.allowBlank && FR.isEmpty(val)) {
            isToastError && this._toastMsg(this.options.errorMsg4Blank || FR.i18nText("Fine-Engine_Base_NOT_NULL_Des"));
            result = false;
        } else {
            if (this._verify(val)) {
                result = true;
            } else {
                if (isToastError) {
                    this._toastMsg(this.errMsg);
                    //REPORT-9820报错后要清掉编辑值
                    this.editor.setValue("");
                }
                result = false;
            }
        }
        this.options.value = val;
        return result;
    },

    getValue: function () {
        var o = this.options;
        if (o.format && o.format !== 'T') {
            return this.options.value;
        }
        var val = this.editor.getValue();
        return this._verify(val) ? val : this.options.value;
    },


    setValue: function (v, verify) {
        v = (v !== undefined && v !== null) ? v + '' : '';
        this.options.value = v;
        if (this.options.format && this.options.format !== 'T' && !this.isEditing) {
            this.editor.setValue(FR.contentFormat(v, this.options.format));
        } else {
            this.editor.setValue(v);
        }
        if (verify === true) {
            this.stopEdit(true);
        }
        this.lastEditedValue = this.getValue();
    },

    setEditingValue: function (v) {
        this.editor.setValue(v);
        this.verify4Style();
        this.lastEditedValue = this.getValue();
    },

    getEditingValue: function () {
        return this.editor.getValue();
    },

    focus: function (noAutoSelect) {
        this.editor.focus(noAutoSelect);
    },

    blur: function () {
        this.editor.blur();
    },

    /**
     * @extends WT.Widget.setEnable()
     */
    setEnable: function (enable) {
        this.options.disabled = !enable;
        this.editor.setEnable(enable);
    },

    setEditorDisplayValue: function (val) {
        this.editor.setValue(val);
    },

    isEditManually: function () {
        return this.editedManually || this.lastEditedValue !== this.getValue();
    },

    clearEditFlag: function () {
        this.lastEditedValue = this.getValue();
        this.editedManually = false;
    }
});

WT.BaseEditor.EVENT_ENTER_PRESSED = 'EVENT_ENTER_PRESSED';
WT.BaseEditor.EVENT_TAB_PRESSED = 'EVENT_TAB_PRESSED';

WT.BaseEditor.PluginRegex = FR.BaseEditor.PluginRegex || [];

WT.Input = FR.extend(BI.Input, {

    _init: function () {
        BI.Input.superclass._init.apply(this, arguments);
        var self = this, o = this.options;
        var ctrlKey = false;
        var inputEventValid = false;
        var _keydown = function (keyCode) {
            self.onKeyDown(keyCode, ctrlKey);
            self._keydown_ = false;
        };
        var _clk = BI.bind(this._click, this);
        this._blurDebounce = BI.debounce(BI.bind(this._blur, this), BI.EVENT_RESPONSE_TIME, true);
        this.element
            .keydown(function (e) {
                inputEventValid = false;
                ctrlKey = e.ctrlKey;
                self.fireEvent(BI.Input.EVENT_QUICK_DOWN, e);
                if (e.keyCode === BI.KeyCode.ENTER) {
                    if (e.ctrlKey) {
                        if (o.supportWrap !== true) {
                            e.preventDefault();
                        }
                    } else {
                        e.preventDefault();
                        e.stopPropagation();
                        self._keydown_ = true;
                        self.fireEvent(WT.Input.EVENT_ENTER_PRESSED);
                        self._keydown_ = false;
                    }
                }
                if (e.keyCode === BI.KeyCode.TAB) {
                    e.preventDefault();
                    e.stopPropagation();
                    self.fireEvent(WT.Input.EVENT_TAB_PRESSED);
                }
            })
            .keyup(function (e) {
                if (e.ctrlKey && e.keyCode == BI.KeyCode.ENTER) {
                    self.element.val(self.getValue() + '\n');
                }
                if (!(inputEventValid && e.keyCode === BI.KeyCode.ENTER)) {
                    self._keydown_ = true;
                    _keydown(e.keyCode);
                }
            })
            .on("input propertychange", function (e) {
                if (!e.keyCode) {
                    return;
                }
                inputEventValid = true;
                self._keydown_ = true;
                _keydown(e.keyCode);
            })
            .click(function (e) {
                e.stopPropagation();
                _clk();
            })
            .mousedown(function (e) {
                self.element.val(self.element.val());
            })
            .focusout(function (e) {
                self._blurDebounce();
            });
        if (FR.Browser.r.safari) {
            this.element.addClass('wt-text-editor-safari-fix');
        }
    },

    setValid: function () {
        BI.Input.superclass.setValid.apply(this, arguments);
        if (this.isValid()) {
            this._lastValidValue = this.getValue();
            this.fireEvent(BI.Input.EVENT_VALID, BI.trim(this.getValue()), this);
        } else {
            if (this._lastValidValue === this.getValue()) {
                this._lastValidValue = null;
            }
            this.fireEvent(BI.Input.EVENT_ERROR, BI.trim(this.getValue()), this);
        }
    },

    onKeyDown: function (keyCode, ctrlKey) {
        if (!this.isValid() || BI.trim(this._lastValidValue) !== BI.trim(this.getValue())) {
            this._checkValidationOnValueChange();
        }
        if (this.isValid() && BI.trim(this.getValue()) !== "") {
            if (BI.trim(this.getValue()) !== this._lastValue && (!this._start || this._lastValue == null || this._lastValue === "")
                || (this._pause === true && !/(\s|\u00A0)$/.test(this.getValue()))) {
                this._start = true;
                this._pause = false;
                this.fireEvent(BI.Controller.EVENT_CHANGE, BI.Events.STARTEDIT, this.getValue(), this);
                this.fireEvent(BI.Input.EVENT_START);
            }
        }
        if (ctrlKey === true && keyCode === 86) {//ctrlKey+V
            this._valueChange();
        } else {
            if (keyCode == BI.KeyCode.SPACE) {
                this.fireEvent(BI.Input.EVENT_SPACE);
            }
            if (keyCode == BI.KeyCode.BACKSPACE && this._lastValue == "") {
                this.fireEvent(BI.Input.EVENT_REMOVE);
            }
            if (keyCode == BI.KeyCode.BACKSPACE || keyCode == BI.KeyCode.DELETE) {
                this.fireEvent(BI.Input.EVENT_BACKSPACE);
            }
        }
        this.fireEvent(BI.Input.EVENT_KEY_DOWN);

        if (BI.isEndWithBlank(this.getValue())) {
            this._pause = true;
            this.fireEvent(BI.Controller.EVENT_CHANGE, BI.Events.PAUSE, "", this);
            this.fireEvent(BI.Input.EVENT_PAUSE);
            this._defaultState();
        } else if ((keyCode === BI.KeyCode.BACKSPACE || keyCode === BI.KeyCode.DELETE) &&
            BI.trim(this.getValue()) === "" && (this._lastValue !== null && BI.trim(this._lastValue) !== "")) {
            this.fireEvent(BI.Controller.EVENT_CHANGE, BI.Events.STOPEDIT, this.getValue(), this);
            this.fireEvent(BI.Input.EVENT_STOP);
            this._valueChange();
        } else {
            this._valueChange();
        }
    },

    focus: function (noAutoSelect) {
        if (!this.element.is(":visible")) {
            // throw new Error("input输入框在不可见下不能focus");
            return;
        }
        if (!this._isEditing === true) {
            this.element.focus();
            this._focus();
            if (noAutoSelect !== true) {
                this.selectAll();
            }
        }
    },

    blur: function () {
        if (!this.element.is(":visible")) {
            // throw new Error("input输入框在不可见下不能blur");
            return;
        }
        if (this._isEditing === true) {
            this._blurDebounce();
        }
    },

    couldUsedAsEditor: function () {
        return true;
    }
});

WT.Input.EVENT_ENTER_PRESSED = 'EVENT_ENTER_PRESSED';
WT.Input.EVENT_TAB_PRESSED = 'EVENT_TAB_PRESSED';