/**
 * 新填报下group类的基类
 * @class WT.BaseGroup
 * @extends WT.Widget
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.autoFit 自适应
 * @cfg {Number} options.columns 展示列数，仅不自适应时有效
 * @cfg {Boolean} options.allowBlank 允许为空
 * @cfg {String} options.errorMsg4Blank 空值时提示信息
 * @cfg {WT.DataSource|Array} options.data 数据源
 *      数据：
 *          [
 *              {"text": "text1", "value": "value1"},
 *              {"text": "text2", "value": "value2"},
 *              {"text": "text3", "value": "value3"}
 *          ]
 */
WT.BaseGroup = FR.extend(WT.Widget, {

    TYPE: 'WT.BaseGroup',

    _defaultConfig: function () {
        return $.extend(WT.BaseGroup.superclass._defaultConfig.apply(), {
            autoFit: true,
            data: [],
            columns: 0,
            allowBlank: true,
            errorMsg4Blank: FR.i18nText('Fine-Engine_Write_Enhanc_Not_Support_Null')
        });
    },

    _initVariable: function () {
        WT.BaseGroup.superclass._initVariable.apply(this);
        var o = this.options, self = this;
        this.data = o.data || [];
        this.items = [];
    },

    _initView: function () {
        WT.BaseGroup.superclass._initView.apply(this);
        var o = this.options, self = this;
        this.element = WT.dom.createJQueryElement('div', 'wt-absolute-layout');
        this.element.css({
            left: o.left,
            top: o.top,
            height: o.height,
            width: o.width,
            overflow: 'auto'
        });

        if (WT.isNotEmptyObject(o.dependence) && o.paraProvider && o.dataSource) {
            this.dataSource = o.dataSource;
            this.dataSource.once(WT.DataSource.EVENT_DATA_UPDATED, function (newData) {
                self.data = newData;
                self._createItems();
            });

            var loadArgs = {};
            var dependence = {};
            for (var k in o.dependence) {
                var paraVal = o.paraProvider.getPara(o.dependence[k]);
                if (!FR.isEmpty(paraVal)) {
                    dependence[WT.isIntString(k) ? o.dependence[k].toUpperCase() : k.toUpperCase()] = paraVal;
                }
            }
            if (!WT.isEmpty(dependence)) {
                loadArgs.dependence = dependence;
            }
            this.dataSource.loadData(loadArgs, JSON.stringify(loadArgs));
            return;
        }
        this._createItems();
    },

    _onDataUpdated: function () {
        //清除子项前清空其监听器，防止闭包造成内存泄漏
        this.items && WT.each(this.items, function (item) {
            item && item.purgeListeners && item.purgeListeners();
        });
        this.items = [];
        this.element.empty();
        this._createItems();
    },

    _createItems: function () {
        var o = this.options, self = this;
        this.rendingItemsData = WT.copyArray(this.data);
        this._beforeItemsCreate();
        WT.each(this.rendingItemsData, function (d, idx) {
            var wrapper = WT.dom.createJQueryElement('span', 'wt-group-item wt-border-box');
            var item = self._createItemComp(d, idx);
            item.element.css('position', 'relative');
            if (!o.autoFit) {
                var columns = o.columns === 0 ? self.rendingItemsData.length : o.columns;
                var rows = Math.ceil(self.rendingItemsData.length / columns);
                var itemWidth = Math.floor(o.width / columns);
                var itemHeight = Math.floor(o.height / rows);
                wrapper.css({
                    display: 'inline-block',
                    height: itemHeight,
                    width: itemWidth
                });
                item.element.css('max-width', '100%');
            }
            item.on(FR.Events.STATECHANGE, function (value) {
                self._onItemValueChange(idx, value);
            });
            WT.dom.appendChild(wrapper, item.element);
            WT.dom.appendChild(self.element, wrapper);
            self.items.push(item);
        });
        this._applyValue();
        this.setEnable(!o.disabled);
        this.setVisible(!o.invisible);
    },

    /**
     * @abstract
     */
    _beforeItemsCreate: function () {
    },

    /**
     * @abstract
     */
    _onItemValueChange: function (idx, value) {
    },

    /**
     * @abstract
     */
    _createItemComp: function (data, idx) {
    },

    /**
     * 将options.value里的值反映到UI组件的状态上
     * @abstract
     */
    _applyValue: function () {
    },

    _isValueEmpty: function () {
        for (var i = 0; i < this.data.length; i++) {
            var item = this.items[i];
            if (item.getValue()) {
                return false;
            }
        }
        return true;
    },

    /**
     * 弹出提示
     * @param msg {String} 提示信息
     */
    _toastMsg: function (msg) {
        FR.Msg.toast(msg);
    },

    setEnable: function (enable) {
        this.options.disabled = !enable;
        WT.each(this.items, function (item) {
            item.setEnable(enable);
        });
    },

    stopEdit: function (isToastError) {
        if (!this.verify()) {
            if (isToastError) {
                this._toastMsg(this.errMsg);
            }
        }
    },

    setValue: function (v) {
        this.options.value = v;
        this._applyValue();
    },
    /**
     * 控件校验(只校验不弹出报错)
     * @returns {boolean}
     */
    verify: function () {
        var o = this.options;
        if (o.allowBlank) {
            return true;
        }
        var isNotEmpty = !this._isValueEmpty();
        if (!isNotEmpty) {
            this.errMsg = this.options.errorMsg4Blank;
        }
        return isNotEmpty;
    }

});