/**
 * 新填报下的单元格标签控件
 * @class WT.CellLabel
 * @extends WT.Widget
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.autoWrap 自动换行
 * @cfg {String} options.vAlign 垂直对齐方式 top/center/bottom
 * @cfg {String} options.align 对齐方式 left/center/right
 * @cfg {String} options.fontFamily 字体
 * @cfg {String} options.fontWeight 字体粗细
 * @cfg {String} options.fontStyle 字体样式
 * @cfg {Number} options.fontSize 字体大小
 * @cfg {Boolean} options.fontShadow 字体阴影
 * @cfg {String} options.decoration 字体修饰（下划线等样式）
 * @cfg {String} options.color 字体颜色
 * @cfg {Boolean} options.fontSubscript 字体下标
 * @cfg {Boolean} options.fontSuperscript 字体上标
 * @cfg {Array} options.hyperlinkGroup 超链接属性
 * @cfg {String} options.format 显示样式
 */
WT.CellLabel = FR.extend(WT.Widget, {

    TYPE: 'WT.CellLabel',

    _defaultConfig: function () {
        return $.extend(WT.CellLabel.superclass._defaultConfig.apply(), {
            value: '',
            display: null,
            watermark: '',
            autoWrap: true,
            vAlign: null,
            align: null,
            fontFamily: null,
            fontWeight: null,
            fontStyle: null,
            fontSize: 12,
            fontShadow: false,
            decoration: null,
            color: 'black',
            fontSubscript: false,
            fontSuperscript: false,
            hyperlinkGroup: null,
            format: null,
            widgetType: null
        });
    },

    _initView: function () {
        WT.CellLabel.superclass._initView.apply(this);
        var o = this.options, self = this;
        var valueObj = o.display;
        if (valueObj) {
            WT.each(
                [
                    'vAlign',
                    'align',
                    'autoWrap',
                    'fontFamily',
                    'fontWeight',
                    'fontStyle',
                    'fontShadow',
                    'fontSubscript',
                    'fontSuperscript',
                    'decoration',
                    'fontSize',
                    'color',
                    'format',
                    'present',
                    'presentDep'
                ], function (item) {
                    if (valueObj[item] !== undefined) {
                        self.options[item] = valueObj[item];
                    }
                }
            );
        }
        this.element = WT.dom.createJQueryElement('div', 'wt-absolute-layout wt-cell-label');
        this.element.css({
            left: o.left,
            top: o.top,
            height: o.height,
            width: o.width,
            'line-height': o.height + 'px'
        });

        this.text = WT.dom.createJQueryElement('div', 'wt-cell-label-text');
        this.text.css({
            'font-size': o.fontSize + 'pt',
            color: o.color,
            width: o.width,
            'line-height': '1.2em',
            'word-wrap': 'break-word',
            overflow: 'visible'
        });
        if (o.align) {
            this.text.css('text-align', o.align);
        }
        if (o.fontFamily) {
            this.text.css('font-family', o.fontFamily);
        }
        if (o.fontWeight) {
            this.text.css('font-weight', o.fontWeight);
        }
        if (o.fontStyle) {
            this.text.css('font-style', o.fontStyle);
        }
        if (o.decoration) {
            this.text.css('text-decoration', o.decoration);
        }
        if (o.fontShadow) {
            this.text.addClass('shadow');
        }
        if (!o.autoWrap) {
            this.text.css('white-space', 'nowrap');
        }
        if (o.vAlign === 'top') {
            this.text.css('vertical-align', 'top');
        }
        else if (o.vAlign === 'bottom') {
            this.text.css('vertical-align', 'bottom');
        }
        else {
            this.text.css('vertical-align', 'middle');
        }
        this._prepareValue();
        this._initHyperlink();
        this._applyValue();

        this.element.append(this.text);

        WT.dom.appendChild(this.element, WT.dom.createElement('i', 'wt-cell-label-placeholder', ' '));

    },

    _initHyperlink: function () {
        var hyperlinks = this.options.hyperlinkGroup;
        if (!hyperlinks || hyperlinks.length === 0) {
            return;
        }
        this.linkWrapper = WT.dom.createJQueryElement('span', null).css('cursor', 'pointer');
        this.linkWrapper.on('click', function (event) {
            FR.doHyperlink(event || window.event, hyperlinks);
        });
        this.text.html(this.linkWrapper);
    },

    _prepareValue: function (settedValue) {
        var o = this.options;
        var valueObj = o.display;
        if (valueObj.displayType === 'image') {
            var src = valueObj.src;
            if (WT.isArray(o.value) && o.value.length === 1) {
                o.value = o.value[0];
            }
            if (!src && o.value && o.value.attach_type === 'image' && o.value.attach_id) {
                src = FR.servletURL
                    + "?op=fr_attach&cmd=ah_download&id="
                    + o.value.attach_id;
            }
            var value = '';
            if (src) {
                value = WT.dom.createJQueryElement('img').attr('src', src);
                value.css({
                    width: o.width,
                    height: o.height
                });
            }
            o.value = value;
        }
        else if (valueObj.displayType === 'html') {
            o.value = valueObj.html;
        }
        else if (valueObj.displayType === 'attachment') {
            var fn = new Function(valueObj.jsAction);
            var link = $('<a href="javascript:void(0)">' + valueObj.fileName + '<a/>').click(function () {
                fn();
            });
            o.value = link;
        }
        else {
            //默认 valueObj.displayType === 'text'
            if (!o.value || (!o.value.attach_id && !(WT.isArray(o.value) && o.value.length > 0 && o.value[0].attach_id))) {
                var v = settedValue !== undefined ? settedValue : valueObj.value;
                if (o.fontSuperscript) {
                    this.text.addClass('super');
                    v = '<sup>' + WT.object2String(v) + '</sup>';
                }
                if (o.fontSubscript) {
                    this.text.addClass('suber');
                    v = '<sub>' + WT.object2String(v) + '</sub>';
                }
                o.value = v;
            }
        }
    },

    _applyValue: function () {
        var o = this.options;
        var value = o.value;

        // value在下面的代码中仅做显示用途，将boolean类型转string
        if (WT.isBoolean(value)) {
            value = value + '';
        }

        if (!WT.isNumber(value) && !value) {
            this._html('')
            if (o.watermark) {
                var waterLabel = $('<div class="wt-cell-label-watermark"/>');
                waterLabel.appendTo(this.text);
                FR.html(waterLabel, WT.replaceSpecialChar4CellDisplay(o.watermark));
            }
            return;
        }

        if (value instanceof Date) {
            value = {
                date_milliseconds: value.getTime()
            }
        }
        if (value.attach_id) {
            value = [value];
        }
        if (WT.isArray(value) && value.length > 0 && value[0].attach_id) {
            var files = WT.map(value, function (v) {
                return {
                    name: v.filename,
                    id: v.attach_id
                };
            });
            var idAll = WT.reduce(WT.map(files, function (file) {
                return file.id;
            }), function (prev, next) {
                return prev + '.' + next;
            });
            value = WT.dom.createJQueryElement('div').css({
                height: o.height,
                width: o.width,
                'line-height': o.height + 'px',
                overflow: 'hidden'
            });
            var fileWrap = $('<div></div>').css({
                display: 'inline-block',
                width: o.width - 17,
                overflow: 'auto'
            }).appendTo(value);
            WT.each(files, function (file) {
                var url = FR.buildServletUrl({
                    op: 'fr_attach',
                    cmd: 'ah_download',
                    id: file.id
                });
                $('<div><a href="' + url + '" target="_blank">' + file.name + '</a></div>').appendTo(fileWrap);
            });
            $('<div class="fr-fileupload-download-all"></div>').css({
                top: 0,
                right: 0
            }).click(function () {
                window.open(FR.buildServletUrl({
                    op: 'fr_attach',
                    cmd: 'ah_download',
                    id: idAll
                }), '_blank');
            }).appendTo(value);
            $('<i class="wt-cell-label-placeholder"/>').css({
                'line-height': o.height + 'px'
            }).html('&nbsp;').appendTo(value);
        }

        var fmt = this.options.format;
        if (value instanceof $) {
            if (this.linkWrapper) {
                this.linkWrapper.html(value);
            } else {
                this.text.html(value);
            }
        }
        else if (value.date_milliseconds) {
            fmt = fmt || 'DMM/dd/yyyy'; //这边只有日期会有默认的fmt
            var v = FR.contentFormat(value.date_milliseconds, fmt);
            this._html(v);
        }
        else if (WT.isString(value) || WT.isNumber(value)) {
            var v = FR.contentFormat(value, fmt);
            this._html(WT.replaceSpecialChar4CellDisplay(v + '').replaceAll('\n', '<br/>'));
        }
        else {
            if (WT.isTreeWidget(o.widgetType)) {
                var jv = value;
                if (WT.isArray(jv)) {
                    var jvs = '';
                    var sep = ';';
                    WT.each(jv, function (jvi, i) {
                        if (i > 0) {
                            jvs += sep;
                        }
                        if (WT.isArray(jvi)) {
                            WT.each(jvi, function (jvj, j) {
                                if (j > 0) {
                                    jvs += ',';
                                }
                                jvs += jvj;
                            })
                        } else {
                            sep = ',';
                            jvs += jvi;
                        }
                    })
                }
                value = jvs;
            }
            this._html(WT.object2String(value));
        }
    },
    _html: function (content) {
        if (this.linkWrapper) {
            this.linkWrapper[0].innerHTML = content;
        } else {
            this.text[0].innerHTML = content;
        }
    },

    /**
     * 设置是否显示校验失败
     * @param invalidMsg {string | null} 为字符串时表示有错误，字符串为错误信息；为null时无错误
     */
    setInvalid: function (invalidMsg) {
        if (!WT.isNullOrUndefined(invalidMsg)) {
            if (!this.warn) {
                this.warn = WT.dom.createJQueryElement('i', 'wt-absolute-layout wt-cell-invalid-icon').attr('title', invalidMsg);
            }
            WT.dom.appendChild(this.text, this.warn);
        }
        else {
            WT.dom.detachFromHTML(this.warn);
        }
    },

    /**
     * 设置值
     * @param v
     */
    setValue: function (v) {
        var oldValue = this.options.value;
        if (v === null || v === undefined) {
            v = ''
        }
        if (v !== oldValue) {
            this._prepareValue(v);
            this._encodeValue(v);
            this._applyValue();
        }
    },

    _encodeValue: function (settedValue) {
        var o = this.options;
        var valueObj = o.display;
        //这边是处理xss注入,做一个转义处理
        if (valueObj.displayType === 'text') {
            //转义只需要处理传过来的值为string或number的情况,其他情况会在_applyValue()中处理
            if(WT.isString(settedValue) || WT.isNumber(settedValue)){
                var templateElement = $('<div>');
                var v = templateElement.text(o.value).html();
                templateElement = null;
                o.value = v;
            }
        }
        //这里是处理当单元格显示方式为html时候控件输入值不生效
        if (valueObj.displayType === 'html') {
            o.value = settedValue !== undefined ? settedValue : valueObj.html;
        }

    },

    getRawHtmlStrValue: function () {
        if (this.linkWrapper) {
            return this.linkWrapper[0].innerHTML;
        } else {
            return this.text[0].innerHTML;
        }
    },

    updateHyperlink: function (hy) {
        if (this.linkWrapper) {
            this.linkWrapper.off('click');
            this.linkWrapper.on('click', function (event) {
                eval(hy);
            });
        }
    },

    destroy: function () {
        this.purgeListeners();
        this.text.off('click');
        this.text.remove();
        if (this.linkWrapper) {
            this.linkWrapper.off('click');
            this.linkWrapper.remove();
        }
        this.element.remove();
        this.text = null;
        this.element = null;
    }

});