/**
 * 新填报下复选框组控件
 * @class WT.CheckBoxGroup
 * @extends WT.BaseGroup
 *
 * @cfg {JSON} options 配置属性
 * @cfg {Boolean} options.autoFit 自适应
 * @cfg {Boolean} options.supportSelectAll 支持全选
 * @cfg {String} options.returnDataType 返回类型 'string' or 'array'
 * @cfg {String} options.returnDataSplit 返回值分隔符 仅返回类型为array时生效
 * @cfg {String} options.returnDataPrefix 返回值起始符 仅返回类型为array时生效
 * @cfg {String} options.returnDataSuffix 返回值结束符 仅返回类型为array时生效
 */
WT.CheckBoxGroup = FR.extend(WT.BaseGroup, {

    TYPE: 'WT.CheckBoxGroup',

    _defaultConfig: function () {
        return $.extend(WT.CheckBoxGroup.superclass._defaultConfig.apply(), {
            supportSelectAll: false,
            returnDataType: 'array',
            returnDataSplit: ',',
            returnDataPrefix: '',
            returnDataSuffix: ''
        });
    },

    _beforeItemsCreate: function () {
        var o = this.options;
        if (o.supportSelectAll) {
            this.rendingItemsData.push({
                text: FR.i18nText('Fine-Engine_Write_Enhanc_Select_All'),
                value: ''
            });
        }
    },

    _onItemValueChange: function (idx, value) {
        var o = this.options;
        if (o.supportSelectAll) {
            if (idx === this.items.length - 1) {
                for (var i = 0; i < this.items.length - 1; i++) {
                    var item = this.items[i];
                    item.setValue(value, true);
                }
            }
            else {
                var selectAllItem = this.items[this.items.length - 1];
                if (this._isAllChecked()) {
                    selectAllItem.setValue(true, true);
                }
                else if (this._isAllUnckecked()) {
                    selectAllItem.setValue(false, true);
                }
                else {
                    selectAllItem.setStateHalf();
                }
            }
        }
        this.options.value = this._generateValue();
        this.fireEvent(FR.Events.STATECHANGE, this.getValue());
    },

    _isAllChecked: function () {
        for (var i = 0; i < this.data.length; i++) {
            var item = this.items[i];
            if (!item.getValue()) {
                return false;
            }
        }
        return true;
    },

    _isAllUnckecked: function () {
        for (var i = 0; i < this.data.length; i++) {
            var item = this.items[i];
            if (item.getValue()) {
                return false;
            }
        }
        return true;
    },

    _createItemComp: function (data, idx) {
        var o = this.options;
        return new WT.CheckBox({
            text: data.text,
            fontSize: o.fontSize,
            height: o.fontSize + 4,
            horizontalPadding: 5
        });
    },

    /**
     * 从UI组件的状态生成值
     * @return {null}
     */
    _generateValue: function () {
        var o = this.options;
        var returnData;
        if (o.returnDataType === 'array') {
            returnData = [];
            for (var i = 0; i < this.data.length; i++) {
                var item = this.items[i];
                if (item.getValue()) {
                    returnData.push(this.data[i].value);
                }
            }
        }
        else {
            returnData = '' + o.returnDataPrefix;
            var valueList = [];
            for (var i = 0; i < this.data.length; i++) {
                var item = this.items[i];
                if (item.getValue()) {
                    valueList.push(this.data[i].value);
                }
            }
            returnData = WT.reduce(valueList, function (prev, curr, idx) {
                var sp = '';
                if (idx > 0) {
                    sp = o.returnDataSplit;
                }
                return prev + sp + curr;
            }, returnData);
            returnData += o.returnDataSuffix;
        }
        return returnData;
    },

    /**
     * 将所有项复位
     * @private
     */
    _clearAllSelect: function () {
        for (var i = 0; i < this.items.length; i++) {
            var item = this.items[i];
            item.setValue(false, true);
        }
    },

    _applyValue: function () {
        var o = this.options, self = this;
        var value = o.value;
        var tmpValue = value;
        if (WT.isNumber(value)) {
            value = value + '';
        }
        if (WT.isString(value)) {
            if (value.indexOf(o.returnDataPrefix) === 0) {
                //去掉前缀
                value = value.substr(o.returnDataPrefix.length);
            }
            if (value.indexOf(o.returnDataSuffix) === value.length - o.returnDataSuffix.length) {
                //去掉后缀
                value = value.substr(0, value.length - o.returnDataSuffix.length);
            }
            //分割成数组
            tmpValue = value.split(o.returnDataSplit);
        }
        if (WT.isArray(tmpValue)) {
            this._clearAllSelect();
            WT.each(tmpValue, function (tVal) {
                for (var i = 0; i < self.data.length; i++) {
                    var dataItem = self.data[i];
                    if (dataItem.value + '' === tVal + '' || dataItem.text + '' === tVal + '') {
                        var compItem = self.items[i];
                        compItem.setValue(true, true);
                        break;
                    }
                }
            });
            if (o.supportSelectAll) {
                var selectAllItem = this.items[this.items.length - 1];
                if (this._isAllChecked()) {
                    selectAllItem.setValue(true, true);
                }
                else if (this._isAllUnckecked()) {
                    selectAllItem.setValue(false, true);
                }
                else {
                    selectAllItem.setStateHalf();
                }
            }
        }
        else {
            this._clearAllSelect();
        }
        o.value = this._generateValue();
    },

    getText: function () {
        var o = this.options;
        var returnData;
        if (o.returnDataType === 'array') {
            returnData = [];
            for (var i = 0; i < this.data.length; i++) {
                var item = this.items[i];
                if (item.getValue()) {
                    returnData.push(this.data[i].text);
                }
            }
        }
        else {
            returnData = '' + o.returnDataPrefix;
            var valueList = [];
            for (var i = 0; i < this.data.length; i++) {
                var item = this.items[i];
                if (item.getValue()) {
                    valueList.push(this.data[i].text);
                }
            }
            returnData = WT.reduce(valueList, function (prev, curr, idx) {
                var sp = '';
                if (idx > 0) {
                    sp = o.returnDataSplit;
                }
                return prev + sp + curr;
            }, returnData);
            returnData += o.returnDataSuffix;
        }
        return returnData;
    }

});